/*
 * @(#)ByteArrayImageInputStream.java  1.0  2008-10-18
 *
 * Copyright (c) 2008 Werner Randelshofer, Immensee, Switzerland.
 * All rights reserved.
 *
 * You may not use, copy or modify this file, except in compliance with the
 * license agreement you entered into with Werner Randelshofer.
 * For details see accompanying license terms.
 */
package ch.randelshofer.io;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.EOFException;
import java.io.IOException;
import java.nio.ByteOrder;
import java.util.Stack;
import javax.imageio.stream.IIOByteBuffer;
import javax.imageio.stream.ImageInputStream;

/**
 * ByteArrayImageInputStream.
 *
 * @author Werner Randelshofer, Hausmatt 10, CH-6405 Immensee
 * @version 1.0 2008-10-18 Created.
 */
public class ByteArrayImageInputStream extends ByteArrayInputStream implements ImageInputStream {

    /**
     * The byte order of the stream as an instance of the enumeration
     * class <code>java.nio.ByteOrder</code>, where
     * <code>ByteOrder.BIG_ENDIAN</code> indicates network byte order
     * and <code>ByteOrder.LITTLE_ENDIAN</code> indicates the reverse
     * order.  By default, the value is
     * <code>ByteOrder.BIG_ENDIAN</code>.
     */
    protected ByteOrder byteOrder = ByteOrder.BIG_ENDIAN;
    /**
     * The current bit offset within the stream.  Subclasses are
     * responsible for keeping this value current from any method they
     * override that alters the bit offset.
     */
    protected int bitOffset;
    /** Length of the buffer used for readFully(type[], int, int) */
    private static final int BYTE_BUF_LENGTH = 8192;
    /** Byte buffer used for readFully(type[], int, int) */
    private byte[] byteBuf = new byte[BYTE_BUF_LENGTH];
    /** The last flushed positione. */
    private long flushedPos;

    private static class Marker {
        private int bytePos, bitPos;
        public Marker(int bytePos, int bitPos) {
            this.bytePos=bytePos;
            this.bitPos=bitPos;
        }
    };

    /** Marked bit and byte positions. */
    private Stack<Marker> markStack = new Stack<Marker>();
    /** The offset of the input stream in the underlying byte array. */
    private int arrayOffset;

    public ByteArrayImageInputStream(byte[] buf) {
        this(buf, ByteOrder.BIG_ENDIAN);
    }

    public ByteArrayImageInputStream(byte[] buf, ByteOrder byteOrder) {
        this(buf, 0, buf.length, byteOrder);
    }

    public ByteArrayImageInputStream(byte[] buf, int offset, int length, ByteOrder byteOrder) {
        super(buf, offset, length);
        arrayOffset = offset;
        this.byteOrder = byteOrder;
    }

//    @Override
    public void setByteOrder(ByteOrder byteOrder) {
	}

//    @Override
    public ByteOrder getByteOrder() {
		return null;
	}

//    @Override
    public void readBytes(IIOByteBuffer buf, int len) throws IOException {
	}

//    @Override
    public boolean readBoolean() throws IOException {
		return false;
	}

//    @Override
    public byte readByte() throws IOException {
		return 0;
	}

//    @Override
    public int readUnsignedByte() throws IOException {
		return 0;
	}

//    @Override
    public short readShort() throws IOException {
		return 0;
	}

//    @Override
    public int readUnsignedShort() throws IOException {
		return 0;
	}

//    @Override
    public char readChar() throws IOException {
		return '0';
	}

//    @Override
    public int readInt() throws IOException {
		return 0;
	}

//    @Override
    public long readUnsignedInt() throws IOException {
		return 0L;
	}

//    @Override
    public long readLong() throws IOException {
		return 0L;
	}

//    @Override
    public float readFloat() throws IOException {
		return 0.0f;
	}

//    @Override
    public double readDouble() throws IOException {
		return 0.0d;
	}

//    @Override
    public String readLine() throws IOException {
		return null;
	}

//    @Override
    public String readUTF() throws IOException {
		return null;
	}

//    @Override
    public void readFully(byte[] b, int off, int len) throws IOException {
	}

//    @Override
    public void readFully(byte[] b) throws IOException {
	}

//    @Override
    public void readFully(short[] s, int off, int len) throws IOException {
	}

//    @Override
    public void readFully(char[] c, int off, int len) throws IOException {
	}

//    @Override
    public void readFully(int[] i, int off, int len) throws IOException {
	}

//    @Override
    public void readFully(long[] l, int off, int len) throws IOException {
	}

//    @Override
    public void readFully(float[] f, int off, int len) throws IOException {
	}

//    @Override
    public void readFully(double[] d, int off, int len) throws IOException {
	}

    private void toShorts(byte[] b, short[] s, int off, int len) {
	}

    private void toChars(byte[] b, char[] c, int off, int len) {
	}

    private void toInts(byte[] b, int[] i, int off, int len) {
	}

    private void toLongs(byte[] b, long[] l, int off, int len) {
	}

    private void toFloats(byte[] b, float[] f, int off, int len) {
	}

    private void toDoubles(byte[] b, double[] d, int off, int len) {
	}

//    @Override
    public long getStreamPosition() {
		return 0L;
	}

//    @Override
    public int getBitOffset() {
		return 0;
	}

//    @Override
    public void setBitOffset(int bitOffset) {
	}

//    @Override
    public int readBit() throws IOException {
		return 0;
	}

//    @Override
    public long readBits(int numBits) throws IOException {
		return 0L;
	}

//    @Override
    public long length() throws IOException {
		return 0L;
	}

//    @Override
    public int skipBytes(int n) throws IOException {
		return 0;
	}

//    @Override
    public long skipBytes(long n) throws IOException {
		return 0L;
	}

//    @Override
    public void seek(long pos) throws IOException {
	}

//    @Override
    public void flushBefore(long pos) throws IOException {
	}

//    @Override
    public void flush() throws IOException {
	}

//    @Override
    public long getFlushedPosition() {
		return 0L;
	}

//    @Override
    public boolean isCached() {
		return false;
	}

//    @Override
    public boolean isCachedMemory() {
		return false;
	}

//    @Override
    public boolean isCachedFile() {
		return false;
	}

    private void checkClosed() {
	}

//    @Override
    public void mark() {
	}

    @Override
    public void reset() {
	}
}
