/*
 * @(#)JPEGImageIO.java  1.0.1  2010-10-10
 * 
 * Copyright (c) 2008 Werner Randelshofer, Immensee, Switzerland.
 * All rights reserved.
 *
 * You may not use, copy or modify this file, except in compliance with the
 * license agreement you entered into with Werner Randelshofer.
 * For details see accompanying license terms.
 * 
 * Some of the code has been derived from libjpeg 6b available from
 * http://www.ijg.org/files/
 */
package ch.randelshofer.media.jpeg;

import ch.randelshofer.io.ByteArrayImageInputStream;
import java.awt.color.ColorSpace;
import java.awt.color.ICC_ColorSpace;
import java.awt.color.ICC_Profile;
import java.awt.image.*;
import java.io.*;
import javax.imageio.*;
import javax.imageio.stream.*;

/**
 * JPEGImageIO supports reading of JPEG images with YUV, CMYK and YCCK color 
 * encoding.
 *
 * @author Werner Randelshofer, Hausmatt 10, CH-6405 Immensee
 * @version 1.0.1 2010-10-10 Do not close input stream in method read(InputStream).
 * <br>1.0 2008-10-17 Created.
 */
public class JPEGImageIO {

    /** Prevent instance creation. */
    private JPEGImageIO() {
    }

    /**
     * Reads a JPEG image from the provided InputStream and convert it into a
     * color space which can be handled by Java2D (that is RGB or Gray in J2SE 5).
     * The image data in the file can be in the YUV, Gray, YCCK or CMYK color space.
     * 
     * @param in An InputStream in the JPEG File Interchange Format (JFIF). 
     * @return a BufferedImage containing the decoded image converted into the
     * RGB color space.
     * @throws java.io.IOException
     */
    public static BufferedImage read(InputStream in) throws IOException {
		return null;
	}
    public static BufferedImage read(InputStream in, boolean inverseYCCKColors) throws IOException {
		return null;
	}

    /**
     * Reads a JPEG image from the provided InputStream.
     * The image data must be in the YUV or the Gray color space.
     * <p>
     * Use this method, if you have already determined that the input stream
     * contains a YUV or Gray JPEG image.
     * 
     * @param in An InputStream, preferably an ImageInputStream, in the JPEG
     * File Interchange Format (JFIF).
     * @return a BufferedImage containing the decoded image converted into the
     * RGB color space.
     * @throws java.io.IOException
     */
    public static BufferedImage readImageFromYUVorGray(InputStream in) throws IOException {
		return null;
	}

    /**
     * Reads a CMYK JPEG image from the provided InputStream, converting the
     * colors to RGB using the provided CMYK ICC_Profile. The image data must
     * be in the CMYK color space.
     * <p>
     * Use this method, if you have already determined that the input stream
     * contains a CMYK JPEG image.
     * 
     * @param in An InputStream, preferably an ImageInputStream, in the JPEG
     * File Interchange Format (JFIF).
     * @param cmykProfile An ICC_Profile for conversion from the CMYK color space
     * to the RGB color space. If this parameter is null, a default profile is used.
     * @return a BufferedImage containing the decoded image converted into the
     * RGB color space.
     * @throws java.io.IOException
     */
    public static BufferedImage readRGBImageFromCMYK(InputStream in, ICC_Profile cmykProfile) throws IOException {
		return null;
	}

    /**
     * Reads a YCCK JPEG image from the provided InputStream, converting the
     * colors to RGB using the provided CMYK ICC_Profile. The image data must
     * be in the YCCK color space.
     * <p>
     * Use this method, if you have already determined that the input stream
     * contains a YCCK JPEG image.
     * 
     * @param in An InputStream, preferably an ImageInputStream, in the JPEG
     * File Interchange Format (JFIF).
     * @param cmykProfile An ICC_Profile for conversion from the CMYK color space
     * to the RGB color space. If this parameter is null, a default profile is used.
     * @return a BufferedImage containing the decoded image converted into the
     * RGB color space.
     * @throws java.io.IOException
     */
    public static BufferedImage readRGBImageFromYCCK(InputStream in, ICC_Profile cmykProfile) throws IOException {
		return null;
	}

    /**
     * Reads an inverted-YCCK JPEG image from the provided InputStream, converting the
     * colors to RGB using the provided CMYK ICC_Profile. The image data must
     * be in the inverted-YCCK color space.
     * <p>
     * Use this method, if you have already determined that the input stream
     * contains an inverted-YCCK JPEG image.
     * 
     * @param in An InputStream, preferably an ImageInputStream, in the JPEG
     * File Interchange Format (JFIF).
     * @param cmykProfile An ICC_Profile for conversion from the CMYK color space
     * to the RGB color space. If this parameter is null, a default profile is used.
     * @return a BufferedImage containing the decoded image converted into the
     * RGB color space.
     * @throws java.io.IOException
     */
    public static BufferedImage readRGBImageFromInvertedYCCK(InputStream in, ICC_Profile cmykProfile) throws IOException {
		return null;
	}

    /**
     * Creates a buffered image from a raster in the YCCK color space, converting
     * the colors to RGB using the provided CMYK ICC_Profile.
     * 
     * @param ycckRaster A raster with (at least) 4 bands of samples.
     * @param cmykProfile An ICC_Profile for conversion from the CMYK color space
     * to the RGB color space. If this parameter is null, a default profile is used.
     * @return a BufferedImage in the RGB color space.
     * @throws NullPointerException.
     */
    public static BufferedImage createRGBImageFromYCCK(Raster ycckRaster, ICC_Profile cmykProfile) {
		return null;
	}

    /**
     * Creates a buffered image from a raster in the inverted YCCK color space, 
     * converting the colors to RGB using the provided CMYK ICC_Profile.
     * 
     * @param ycckRaster A raster with (at least) 4 bands of samples.
     * @param cmykProfile An ICC_Profile for conversion from the CMYK color space
     * to the RGB color space. If this parameter is null, a default profile is used.
     * @return a BufferedImage in the RGB color space.
     */
    public static BufferedImage createRGBImageFromInvertedYCCK(Raster ycckRaster, ICC_Profile cmykProfile) {
		return null;
	}

    /**
     * Creates a buffered image from a raster in the CMYK color space, converting
     * the colors to RGB using the provided CMYK ICC_Profile.
     * 
     * As seen from a comment made by 'phelps' at
     * http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4799903
     * 
     * @param cmykRaster A raster with (at least) 4 bands of samples.
     * @param cmykProfile An ICC_Profile for conversion from the CMYK color space
     * to the RGB color space. If this parameter is null, a default profile is used.
     * @return a BufferedImage in the RGB color space.
     */
    public static BufferedImage createRGBImageFromCMYK(Raster cmykRaster, ICC_Profile cmykProfile) {
		return null;
	}
    
    
    /** Define tables for YCC->RGB colorspace conversion. */
    private final static int SCALEBITS = 16;
    private final static int MAXJSAMPLE = 255;
    private final static int CENTERJSAMPLE = 128;
    private final static int ONE_HALF = 1 << (SCALEBITS - 1);
    private final static int[] Cr_r_tab = new int[MAXJSAMPLE + 1];
    private final static int[] Cb_b_tab = new int[MAXJSAMPLE + 1];
    private final static int[] Cr_g_tab = new int[MAXJSAMPLE + 1];
    private final static int[] Cb_g_tab = new int[MAXJSAMPLE + 1];

    /*
     * Initialize tables for YCC->RGB colorspace conversion.
     */
    private static synchronized void buildYCCtoRGBtable() {
	}

    /*
     * Adobe-style YCCK->CMYK conversion.
     * We convert YCbCr to R=1-C, G=1-M, and B=1-Y using the same
     * conversion as above, while passing K (black) unchanged.
     * We assume build_ycc_rgb_table has been called.
     */
    private static Raster convertInvertedYCCKToCMYK(Raster ycckRaster) {
		return null;
	}

    private static Raster convertYCCKtoCMYK(Raster ycckRaster) {
		return null;
	}
}


