/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2016 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.enablement.resource.model.api;

import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.jcr.RepositoryException;

import org.apache.jackrabbit.api.security.user.Group;
import org.apache.sling.api.resource.PersistenceException;
import org.apache.sling.commons.json.JSONArray;

import com.adobe.cq.social.enablement.exceptions.EnablementException;
import com.adobe.cq.social.scf.SocialComponent;
import com.day.cq.commons.jcr.JcrConstants;

public interface EnablementResourceModel extends SocialComponent {

    public static final String RESOURCE_TYPE = "social/enablement/components/hbs/model/resource";
    public static final String ENABLEMENT_TYPE_VALUE = "resource";

    public static final String JCR_TITLE = JcrConstants.JCR_TITLE;
    public static final String RESOURCE_NAME = "se_resource-name";
    public static final String DESCRIPTION = "se_description";
    public static final String RESOURCE_ID = "se_resource-id";
    public static final String RESOURCE_UID = "se_resource-uid";
    public static final String RESOURCE_TAGS = "se_resource-tags";
    public static final String ENABLEMENT_TYPE = "se_enablement-type";
    public static final String COVER_IMAGE = "se_cover-image";
    public static final String SITE = "se_site";
    public static final String DUE_DATE = "se_due-date";
    public static final String PUBLISHED = "se_published";
    public static final String FIRST_PUBLISHED_BY = "se_first-published-by";
    public static final String ALLOW_COMMENTS = "se_allow-comments";
    public static final String ALLOW_RATINGS = "se_allow-ratings";
    public static final String SHOW_IN_CATALOG = "se_show-in-catalog";
    public static final String ANONYMOUS_ACCESS = "se_anonymous-access";
    public static final String AUTHORS = "se_authors";
    public static final String CONTACTS = "se_contacts";
    public static final String EXPERTS = "se_experts";
    public static final String ASSIGNEES = "se_assignees";
    public static final String DATE_MODIFIED = "se_date-modified";
    public static final String DATE_CREATED = "se_date-created";
    public static final String DATE_PUBLISHED = "se_date-published";
    public static final String CARD_IMAGE_PATH = "se_card-image";
    public static final String ENROLLMENT_RESOURCE_NAME = "se_enrollment";
    public static final String REPLICATION_ACTION = "se_replication-action";
    public static final String APPLY_TO_LIST = "se_:applyTo";
    public static final String DIRECT_ENROLLMENT_QUERY_PROPERTY = "directly-enrolled";

    // From the site resource
    public static final String TAG_NAMESPACES = "tagNameSpaces";

    // the group of users who are directly enrolled in the resource
    public static final String DIRECT_ENROLLEE_GROUP_ID = "direct-enrollees";

    // the group of users who are indirectly enrolled in the resource
    public static final String INDIRECT_ENROLLEE_GROUP_ID = "indirect-enrollees";

    // the group of users who have a report record (some might have interacted with the resource though not enrolled)
    public static final String REPORT_USERS_GROUP_ID = "report-users";

    /**
     * @return resource title
     */
    public String getTitle() {
		return null;
	}

    /**
     * @return resource name
     */
    public String getName() {
		return null;
	}

    /**
     * updates resource name
     */
    public void setName(String name) throws PersistenceException, EnablementException {
	}

    /**
     * @return resource description
     */
    public String getDescription() {
		return null;
	}

    /**
     * updates resource description
     */
    public void setDescription(String description) throws PersistenceException, EnablementException {
	}

    /**
     * @return resource enablement type
     */
    public String getEnablementType() {
		return null;
	}

    /**
     * @return resource cover image
     */
    public String getCoverImage() {
		return null;
	}

    /**
     * @return resource site path
     */
    public String getSite() {
		return null;
	}

    /**
     * @return resource site id
     */
    public String getSiteId() {
		return null;
	}

    /**
     * @return resource site unique id
     */
    public String getSiteUID() {
		return null;
	}

    /**
     * @return resource site language
     */
    public String getLanguage() {
		return null;
	}

    /**
     * @return resource date
     */
    public String getFriendlyDueDate() {
		return null;
	}

    /**
     * @return resource date as a string
     */
    public String getDueDateAsString() {
		return null;
	}

    /**
     * @return resource date as a calendar
     */
    public Calendar getDueDateAsCalendar() {
		return null;
	}

    /**
     * @return resource date
     */
    public Date getDueDate() {
		return null;
	}

    /**
     * @return resource id
     */
    public String getResourceId() {
		return null;
	}

    /**
     * @return resource id
     */
    public String getResourceUID() {
		return null;
	}

    /**
     * Gets the list of tagIds associated with the given Enablement content.
     * @return list of tagIds
     * @throws RepositoryException on failure
     */
    public String[] getResourceTags() throws RepositoryException {
		return null;
	}

    /**
     * @return resource allow comments
     */
    public Boolean getAllowComments() {
		return null;
	}

    /**
     * @return resource allow ratings
     */
    public Boolean getAllowRatings() {
		return null;
	}

    /**
     * @return resource is published
     */
    public Boolean getIsPublished() {
		return null;
	}

    /**
     * @return resource allow anonymous
     */
    public Boolean getAllowAnonymous() {
		return null;
	}

    /**
     * @return allow catalog visibility
     */
    Boolean getAllowCatalogVisibility() {
		return null;
	}

    /**
     * @param key
     * @param assetMap
     * @throws PersistenceException
     * @throws EnablementException
     */
    public void createNamedAsset(String key, Map<String, Object> assetMap) throws PersistenceException,
        EnablementException {
	}

    /**
     * Create social resource endpoints.
     * @param displayName
     * @throws PersistenceException
     * @throws EnablementException
     */
    public void setSocialResources(final String displayName, final Boolean allowComments, final Boolean allowRatings)
        throws PersistenceException, EnablementException {
	}

    /**
     * @param key
     * @param authorizableMap
     * @throws PersistenceException
     * @throws EnablementException
     */
    public void setAuthorizables(String key, Map<String, Object> authorizableMap) throws PersistenceException,
        EnablementException {
	}

    /**
     * Returns an array of resource assets.
     * @return
     */
    public List<EnablementAssetModel> getAssets() {
		return null;
	}

    /**
     * Returns an object representing the primary resource asset.
     * @return
     */
    public EnablementAssetModel getPrimaryAsset() {
		return null;
	}

    /**
     * Returns an object representing the primary resource asset.
     * @return
     */
    public EnablementAssetModel getSmallAsset() {
		return null;
	}

    /**
     * Returns an object representing the primary resource asset.
     * @return
     */
    public EnablementAssetModel getLargeAsset() {
		return null;
	}

    /**
     * Returns an object representing the primary resource asset.
     * @return
     */
    public EnablementAssetModel getFragmentAsset() {
		return null;
	}

    /**
     * @return get the enablement card image
     */
    public EnablementCardImage getEnablementCardImage() {
		return null;
	}

    /**
     * @throws EnablementException
     */
    void setCoverImagePath(final String coverImagePath) throws EnablementException {
	}

    /**
     * @throws PersistenceException
     * @throws EnablementException
     */
    public void setCardImage(final String cardImageTmpPath) throws PersistenceException, EnablementException {
	}

    /**
     * Set due date for a resourse
     * @param dueDate
     * @throws EnablementException
     */
    public void setDueDate(final Calendar dueDate) throws EnablementException {
	}

    /**
     * @return an object representing the social resource endpoints
     */
    public EnablementSocialModel getSocial() {
		return null;
	}

    /**
     * Returns an object representing the contacts
     * @return
     */
    public EnablementAuthorizables getContacts() {
		return null;
	}

    /**
     * Returns an object representing the authors
     * @return
     */
    public EnablementAuthorizables getAuthors() {
		return null;
	}

    /**
     * Returns an object representing the experts
     * @return
     */
    public EnablementAuthorizables getExperts() {
		return null;
	}

    /**
     * Returns an object representing the assignees
     * @return
     */
    public EnablementAuthorizables getAssignees() {
		return null;
	}

    /**
     * Returns a the primary asset type
     * @return
     */
    public String getPrimaryAssetType() {
		return null;
	}

    /**
     * Returns the first published date
     * @return
     */
    public Date getFirstPublishedDate() {
		return null;
	}

    /**
     * @param groupId
     * @return the authorizable group for the resource with the corresponding group id
     */
    public Group getResourceGroup(final String groupId) {
		return null;
	}

    /**
     * @param name
     * @param primaryAssetArray
     */
    void importNamedAsset(String name, JSONArray primaryAssetArray) {
	}

    /**
     * @param tags
     * @throws EnablementException
     */
    public void setTags(String[] tags) throws EnablementException {
	}

    /**
     * @param showInCatalog
     * @throws EnablementException
     */
    public void setShowInCatalog(Boolean showInCatalog) throws EnablementException {
	}

    /**
     * @param showInCatalog
     * @throws EnablementException
     */
    public void setAnonymousAccess(Boolean allowAnonymousAccess) throws EnablementException {
	}
}
