/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.moderation.dashboard.api;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Dictionary;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Property;
import org.apache.felix.scr.annotations.Reference;
import org.apache.felix.scr.annotations.ReferenceCardinality;
import org.apache.felix.scr.annotations.ReferencePolicy;
import org.apache.felix.scr.annotations.Service;
import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.resource.Resource;
import org.osgi.service.cm.ConfigurationAdmin;
import org.osgi.service.component.ComponentContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.cq.social.moderation.dashboard.impl.FilterGroupImpl;
import com.adobe.cq.social.scf.ClientUtilities;
import com.adobe.cq.social.scf.QueryRequestInfo;
import com.adobe.cq.social.scf.SocialComponent;
import com.adobe.cq.social.scf.core.AbstractSocialComponentFactory;

@Component(metatype = true, label = "AEM Communities Moderation Dashboard Filters",
        description = "This component holds a group" + " of search filters.", immediate = true)
@Service
public class FilterGroupSocialComponentFactory extends AbstractSocialComponentFactory {

    /** Lookup used to find configuration properties. */
    public static final String SERVICE_PID =
        "com.adobe.cq.social.moderation.dashboard.api.FilterGroupSocialComponentFactory";

    /**
     * Default resourceTypeNames. * The i18n.get statements are required for string translation, please add for new
     * names
     */
    public static final String DEFAULT_QNA_QUESTION = "QnA Question"; // i18n.get("QnA Question")
    public static final String DEFAULT_QNA_ANSWER = "QnA Answer"; // i18n.get("QnA Answer")
    public static final String DEFAULT_FORUM_TOPIC = "Forum Topic"; // i18n.get("Forum Topic")
    public static final String DEFAULT_FORUM_REPLY = "Forum Reply"; // i18n.get("Forum Reply")
    public static final String DEFAULT_COMMENT = "Comment"; // i18n.get("Comment")
    public static final String DEFAULT_FILE = "File Library Document"; // i18n.get("File Library Document")
    public static final String DEFAULT_FOLDER = "File Library Folder"; // i18n.get("File Library Folder")
    public static final String DEFAULT_BLOGS = "Blog Article"; // i18n.get("Blog Article")
    public static final String DEFAULT_BLOGS_COMMENTS = "Blog Comment"; // i18n.get("Blog Comment")
    public static final String DEFAULT_CALENDAR_EVENTS = "Calendar Event"; // i18n.get("Calendar Event")
    public static final String DEFAULT_CALENDAR_EVENT_COMMENTS = "Calendar Comment"; // i18n.get("Calendar Comment")
    public static final String DEFAULT_IDEATION_IDEA = "Idea"; // i18n.get("Idea")
    public static final String DEFAULT_IDEATION_COMMENT = "Ideation Comment"; // i18n.get("Ideation Comment")

    /** Default resourceTypes. */
    public static final String DEFAULT_QNA_QUESTION_RT = "social/qna/components/topic";
    public static final String DEFAULT_QNA_QUESTION_HBS_RT = "social/qna/components/hbs/topic";
    public static final String DEFAULT_QNA_ANSWER_RT = "social/qna/components/post";
    public static final String DEFAULT_QNA_ANSWER_HBS_RT = "social/qna/components/hbs/post";
    public static final String DEFAULT_FORUM_TOPIC_RT = "social/forum/components/topic";
    public static final String DEFAULT_FORUM_TOPIC_HBS_RT = "social/forum/components/hbs/topic";
    public static final String DEFAULT_FORUM_REPLY_RT = "social/forum/components/post";
    public static final String DEFAULT_FORUM_REPLY_HBS_RT = "social/forum/components/hbs/post";
    public static final String DEFAULT_COMMENT_RT = "social/commons/components/comments/comment";
    public static final String DEFAULT_COMMENT_HBS_RT = "social/commons/components/hbs/comments/comment";
    public static final String DEFAULT_FILE_HBS_RT = "social/filelibrary/components/hbs/document";
    public static final String DEFAULT_FOLDER_HBS_RT = "social/filelibrary/components/hbs/folder";
    public static final String DEFAULT_BLOGS_HBS_RT = "social/journal/components/hbs/entry_topic";
    public static final String DEFAULT_BLOGS_COMMENTS_HBS_RT = "social/journal/components/hbs/comment";
    public static final String DEFAULT_CALENDAR_EVENTS_HBS_RT = "social/calendar/components/hbs/event";
    public static final String DEFAULT_CALENDAR_EVENT_COMMENTS_HBS_RT =
        "social/calendar/components/hbs/event_comment";
    public static final String DEFAULT_IDEATION_HBS_RT = "social/ideation/components/hbs/idea";
    public static final String DEFAULT_IDEATION_COMMENT_HBS_RT = "social/ideation/components/hbs/comment";

    /** Name used for OSGI configuration of resourceTypes. */
    @Property(label = "ResourceType Filters", value = {"", ""},
            description = "Specify resource types for searching: ScreenName=resource/type1:resource/type2... etc.")
    public static final String RT_FILTERS = "resourceType.filters";

    /** Pattern for parsing configuration properties. */
    public static final String CONFIG_PARSE_PATTERN = "=|:";

    /** Prefix for parsing filterGroup configuration properties. */
    public static final String CONFIG_FILTER_GROUP_PREFIX = "filterGroup.";

    /** Default contentTypes map, only in case the configuration lookup fails. */
    public static final Map<String, List<String>> DEFAULT_CONTENT_TYPES_MAP =
        new LinkedHashMap<String, List<String>>(5);
    static {
        DEFAULT_CONTENT_TYPES_MAP.put(DEFAULT_COMMENT, Arrays.asList(DEFAULT_COMMENT_RT, DEFAULT_COMMENT_HBS_RT));
        DEFAULT_CONTENT_TYPES_MAP.put(DEFAULT_FORUM_TOPIC,
            Arrays.asList(DEFAULT_FORUM_TOPIC_RT, DEFAULT_FORUM_TOPIC_HBS_RT));
        DEFAULT_CONTENT_TYPES_MAP.put(DEFAULT_FORUM_REPLY,
            Arrays.asList(DEFAULT_FORUM_REPLY_RT, DEFAULT_FORUM_REPLY_HBS_RT));
        DEFAULT_CONTENT_TYPES_MAP.put(DEFAULT_QNA_QUESTION,
            Arrays.asList(DEFAULT_QNA_QUESTION_RT, DEFAULT_QNA_QUESTION_HBS_RT));
        DEFAULT_CONTENT_TYPES_MAP.put(DEFAULT_QNA_ANSWER,
            Arrays.asList(DEFAULT_QNA_ANSWER_RT, DEFAULT_QNA_ANSWER_HBS_RT));
        DEFAULT_CONTENT_TYPES_MAP.put(DEFAULT_BLOGS, Arrays.asList(DEFAULT_BLOGS_HBS_RT));
        DEFAULT_CONTENT_TYPES_MAP.put(DEFAULT_BLOGS_COMMENTS, Arrays.asList(DEFAULT_BLOGS_COMMENTS_HBS_RT));
        DEFAULT_CONTENT_TYPES_MAP.put(DEFAULT_CALENDAR_EVENTS, Arrays.asList(DEFAULT_CALENDAR_EVENTS_HBS_RT));
        DEFAULT_CONTENT_TYPES_MAP.put(DEFAULT_CALENDAR_EVENT_COMMENTS,
            Arrays.asList(DEFAULT_CALENDAR_EVENT_COMMENTS_HBS_RT));
        DEFAULT_CONTENT_TYPES_MAP.put(DEFAULT_FOLDER, Arrays.asList(DEFAULT_FOLDER_HBS_RT));
        DEFAULT_CONTENT_TYPES_MAP.put(DEFAULT_FILE, Arrays.asList(DEFAULT_FILE_HBS_RT));
        DEFAULT_CONTENT_TYPES_MAP.put(DEFAULT_IDEATION_IDEA, Arrays.asList(DEFAULT_IDEATION_HBS_RT));
        DEFAULT_CONTENT_TYPES_MAP.put(DEFAULT_IDEATION_COMMENT, Arrays.asList(DEFAULT_IDEATION_COMMENT_HBS_RT));
    }

    /** Logger. */
    private static final Logger LOG = LoggerFactory.getLogger(FilterGroupSocialComponentFactory.class);

    /** Config admin to read resource types from config. */
    @Reference(cardinality = ReferenceCardinality.MANDATORY_UNARY, policy = ReferencePolicy.STATIC)
    private ConfigurationAdmin configurationAdmin;

    /** List of resourceType filters. */
    private Map<String, List<String>> resourceTypeFilters;

    @Override
    public SocialComponent getSocialComponent(final Resource resource) {
		return null;
	}

    @Override
    public SocialComponent getSocialComponent(final Resource resource, final SlingHttpServletRequest request) {
		return null;
	}

    @Override
    public SocialComponent getSocialComponent(final Resource resource, final ClientUtilities clientUtils,
        final QueryRequestInfo queryRequestInfo) {
			return null;
		}

    @Override
    public String getSupportedResourceType() {
		return null;
	}

    /**
     * Return the resourceType filter names in a List. This method can be used by other classes that need access to
     * the same data.
     * @return a List of resourceType filter names
     */
    private List<String> getResourceTypeFilterNames() {
		return null;
	}

    /**
     * Return the resourceType filters in a Map. This method can be used by other classes that need access to the same
     * data.
     * @return a List of resourceType filter names
     */
    public Map<String, List<String>> getResourceTypeFilters() {
		return null;
	}

    /**
     * Re-create the resourceTypeFilters map and pull the values from the OSGI config.
     * @param ComponentContext context Used when the component is activated. Can be null and the configuration will be
     *            taken using the configurationAdmin.
     * @return a List of resourceType filter names
     */
    private void setResourceTypeFilters(final ComponentContext context) {
	}

    /**
     * OSGI Activate.
     * @param context the ComponentContext sent by OSGI.
     */
    @Override
    protected void activate(final ComponentContext context) {
	}
}
