/*******************************************************************************
 * ADOBE CONFIDENTIAL
 * __________________
 *
 * Copyright 2016 Adobe Systems Incorporated
 * All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 ******************************************************************************/

package com.adobe.cq.xf.ui;

import aQute.bnd.annotation.ProviderType;
import com.day.cq.wcm.api.Page;

import javax.jcr.RepositoryException;
import java.util.Calendar;
import java.util.Set;

/**
 * The administration console item for experience fragments.
 */
@ProviderType
public interface ExperienceFragmentConsoleItem {

    /**
     * Returns the path of the experience fragment.
     *
     * @return String
     */
    String getPath() {
		return null;
	}

    /**
     * Returns the resource type of the experience fragment. Can be the original cq/experience-fragments/components/xfpage or an overlay.
     *
     * @return String
     */
    String getResourceType() {
		return null;
	}

    /**
     * Returns the title of the experience fragment page.
     * @return String
     */
    String getTitle() {
		return null;
	}

    /**
     * Returns the last modification date of the resource.
     *
     * @return {@link Calendar} object
     */
    Calendar getLastModified() {
		return null;
	}

    /**
     * Returns the name of the user that modified this resource.
     *
     * @return String
     */
    String getLastModifiedBy() {
		return null;
	}

    /**
     * Returns the date that this resource was published (activated).
     *
     * @return a {@link Calendar} object or <code>null</code> if the page is not published
     */
    Calendar getPublishedDate() {
		return null;
	}

    /**
     * Returns a set of relation identifiers for the quick actions.
     *
     * @return Set of relation identifiers
     */
    Set<String> getQuickactionsRel() throws RepositoryException {
		return null;
	}

    /**
     * Returns the URL for the "Properties" action. This URL is determined according to the resource type.
     *
     * @return the "href" property for the "Properties" action.
     */
    String getViewPropertiesHref() {
		return null;
	}

    /**
     * Checks whether the current item is a folder or not.
     *
     * @return <code>true</code> if the resource of this item is a folder, and <code>false</code> otherwise.
     */
    boolean isFolder() {
		return false;
	}

    /**
     * Checks whether the current resource is a live-copy or not.
     *
     * @return <code>true</code> if the page is a live-copy, and <code>false</code> otherwise.
     */
    boolean isLiveCopy() {
		return false;
	}

    /**
     * Checks whether the page represented by this item was created in the last 24hrs.
     *
     * @return <code>true</code> if the page was created in the last 24hrs, <code>false</code> otherwise
     */
    boolean isNew() {
		return false;
	}

    /**
     * Checks whether this resource is an experience fragment "root" page and not a variation.
     *
     * @return
     */
    boolean isExperienceFragmentRoot() {
		return false;
	}

    /**
     * Returns the parent Experience Fragment for this variation
     * @return the {@link Page} which is the Experience Fragment if the current page is a variation or {@code null} otherwise
     */
    Page getParentXf() {
		return null;
	}

    /**
     * Determines the URL that the user navigates when clicking the item.
     *
     * @return A valid URL, or an empty string if the item represents an experience fragment variant.
     */
    String getNavigatorHref() {
		return null;
	}

    /**
     * Checks if the experience fragment is exported as an Adobe Target offer.
     * @return {@code true} if the variation is exported to Adobe Target, {@code false} otherwise
     */
    boolean isExportedToAdobeTarget() {
		return false;
	}
}