/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.granite.httpcache.api;

import org.apache.commons.lang.time.FastDateFormat;

import java.io.ByteArrayOutputStream;
import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.TimeZone;

/**
 * Container class for HTTP headers.
 */
public class Headers {

    private final List<Entry> entries = new ArrayList<Entry>(16);

    /** GMT Timezone. */
    private static final TimeZone GMT = TimeZone.getTimeZone("GMT");
    
    /** Date formatter. */
    private static final FastDateFormat RFC1123 =
            FastDateFormat.getInstance("EEE, dd MMM yyyy HH:mm:ss z", GMT, Locale.US);

    /**
     * A header entry.
     */
    public static class Entry {
        
        public static enum Type {
            STRING, LONG, INT
        };
        
        final String name;
        final Type type;
        private String s;
        private long l;
        private int i;
        
        Entry(String name, String value) {
            this.type = Type.STRING;
            this.name = name;
            this.s = value;
        }

        Entry(String name, long value) {
            this.type = Type.LONG;
            this.name = name;
            this.l = value;
        }
        
        Entry(String name, int value) {
            this.type = Type.INT;
            this.name = name;
            this.i = value;
        }
        
        /**
         * Return the string representation of this entry's value. If the value's
         * type is not <code>STRING</code>, it is converted to a string as follows:
         * <ul>
         * <li>a <b>long</b> is formatted as a date in RFC1123 format</li>
         * <li>an <b>integer</b> is formatted as decimal number</li>
         * </ul>
         * 
         * @return string representation
         */
        public String getString() {
			return null;
		}
        
        /**
         * Return a long value if the type is LONG, otherwise <code>-1</code>
         * 
         * @return long
         */
        public long getLong() {
			return 0L;
		}

        /**
         * Return an integer value if the type is INT, otherwise <code>0</code>
         * 
         * @return integer
         */
        public int getInt() {
			return 0;
		}

        /**
         * Return the header type.
         *
         * @return type
         */
        public Type getType() {
			return null;
		}

        /**
         * Return the header name.
         *
         * @return name
         */
        public String getName() {
			return null;
		}

        @Override
        public String toString() {
			return null;
		}
    }
    
    /**
     * Return the first header matching a given name.
     * 
     * @param name
     *            name
     * @return value of header or <code>null</code>
     */
    public String getHeader(String name) {
		return null;
	}

    /**
     * Return all headers matching a given name.
     * 
     * @param name name
     * @return matching values or <code>null</code>
     */
    public String[] getHeaders(String name) {
		return null;
	}
    
    /**
     * Return the first header matching a given name.
     * 
     * @param name name
     * @return value of date header or <code>-1</code>
     */
    public long getDateHeader(String name) {
		return 0L;
	}

    /**
     * Return the first header matching a given name.
     * 
     * @param name name
     * @return value of date header or <code>0</code>
     */
    public int getIntHeader(String name) {
		return 0;
	}
    
    /**
     * Set a header. This will replace the first existing entry or add
     * a new one.
     * 
     * @param name name
     * @param value value, if <code>null</code> remove an existing header
     */
    public void setHeader(String name, String value) {
	}

    /**
     * Set a header. This will replace the first existing entry or add
     * a new one.
     * 
     * @param name name
     * @param value value
     */
    public void setHeader(String name, long value) {
	}

    /**
     * Set a header. This will replace the first existing entry or add
     * a new one.
     * 
     * @param name name
     * @param value value
     */
    public void setHeader(String name, int value) {
	}
    
    private void setHeader(Entry e) {
	}
    
    /**
     * Add a header.
     * 
     * @param name name
     * @param value value
     */
    public void addHeader(String name, String value) {
	}

    /**
     * Add a header.
     * 
     * @param name name
     * @param value value
     */
    public void addHeader(String name, long value) {
	}

    /**
     * Add a header.
     * 
     * @param name name
     * @param value value
     */
    public void addHeader(String name, int value) {
	}
    
    /**
     * Return all entries.
     * 
     * @return entries
     */
    public Entry[] getEntries() {
		return null;
	}
    
    //----------------------------------------------------------- Serialization

    // TODO: should the serialization format be customizable?
    
    /**
     * Store headers to an output stream.
     * 
     * @param out output stream
     * @throws java.io.IOException if an I/O error occurs
     */
    public void save(OutputStream out) throws IOException {
	}

    private static void writeShort(OutputStream out, int n) throws IOException {
	}

    private static void writeString(OutputStream out, String s) throws IOException {
	}

    private static void writeInt(OutputStream out, int n) throws IOException {
	}

    private static void writeLong(OutputStream out, long l) throws IOException {
	}

    /**
     * Load stored headers from an input stream.
     *
     * @param in input stream
     * @throws java.io.IOException if an I/O error occurs
     */
    public void load(InputStream in) throws IOException {
	}
    
    private static int readUnsignedShort(InputStream in) throws IOException {
		return 0;
	}
    
    private static int readInt(InputStream in) throws IOException {
		return 0;
	}
    
    private static long readLong(InputStream in) throws IOException {
		return 0L;
	}
    
    private static String readString(InputStream in) throws IOException {
		return null;
	}
    
    @Override
    public String toString() {
		return null;
	}
}
