/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.granite.maintenance;

import java.util.Calendar;

import org.osgi.annotation.versioning.ProviderType;


/**
 * Description of a maintenance task
 */
@ProviderType
public interface MaintenanceTaskInfo {

    enum TaskState {
        UNKNOWN,
        SUCCEEDED,
        FAILED,
        STOPPED
    }

    enum TaskSchedule {
        DAILY,
        WEEKLY,
        @Deprecated
        BIWEEKLY,
        MONTHLY
    }

    /**
     * The unique name of a maintenance task.
     */
    String getName() {
		return null;
	}

    /**
     * Whether a task supports stopping
     */
    boolean isStoppable() {
		return false;
	}

    /**
     * Whether a task is conservative.
     */
    boolean isConservative() {
		return false;
	}

    /**
     * Whether a task supports throttling.
     */
    boolean supportsThrottling() {
		return false;
	}

    /**
     * A human readable title.
     */
    String getTitle() {
		return null;
	}

    /**
     * A human readable description
     */
    String getDescription() {
		return null;
	}

    /**
     * Get the last date this task has been run.
     */
    Calendar getLastRunTime() {
		return null;
	}

    /**
     * Get the last date this task has succeeded or <code>null</code>.
     * See {@link TaskState#SUCCEEDED}
     */
    Calendar getLastSuccessTime() {
		return null;
	}

    /**
     * Get the state of the last run.
     */
    TaskState getLastRunState() {
		return null;
	}

    /**
     * Whether the task is currently running.
     */
    boolean isRunning() {
		return false;
	}

    /**
     * This method always returns {@link TaskSchedule#DAILY}
     * @deprecated Use {@link #getSuggestedSchedule()}
     */
    @Deprecated
    TaskSchedule getSchedule() {
		return null;
	}

    /**
     * Get the suggested schedule.
     * @return The suggested schedule or <code>null</code>
     */
    TaskSchedule getSuggestedSchedule() {
		return null;
	}

    /**
     * Return the topic for the task.
     */
    String getTaskTopic() {
		return null;
	}

    /**
     * Whether this task is considered to be mandatory.
     */
    boolean isMandatory() {
		return false;
	}

    /**
     * @return - the task's configuration PID, to be used for configuration through the UI.
     * The method returns null if there's any problem with retrieving the PID
     */
    String getConfigurationPid() {
		return null;
	}
}
