/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.granite.rest.converter.siren;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.jackrabbit.util.ISO8601;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ValueMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.granite.haf.apimodel.impl.ApiModelWrapper;
import com.adobe.granite.haf.apimodel.internal.FilterableChildren;
import com.adobe.granite.rest.Constants;
import com.adobe.granite.rest.converter.ResourceConverter;
import com.adobe.granite.rest.converter.ResourceConverterContext;
import com.adobe.granite.rest.converter.ResourceConverterException;
import com.adobe.granite.rest.filter.Filter;
import com.adobe.granite.rest.utils.Resources;
import com.adobe.granite.rest.utils.URIUtils;
import com.adobe.reef.siren.Action;
import com.adobe.reef.siren.Entity;
import com.adobe.reef.siren.Link;
import com.adobe.reef.siren.builder.BuilderException;
import com.adobe.reef.siren.builder.EntityBuilder;
import com.adobe.reef.siren.builder.LinkBuilder;

/**
 * {@code AbtractConverter} is a base implementation of {@link ResourceConverter}. ResourceConverter implementations
 * are encouraged to extend from this abstract base class.
 */
public abstract class AbstractSirenConverter implements ResourceConverter<Entity> {

    /**
     * Allowed property prefixes for short version
     */
    // TODO: make this list configurable?
    protected static final String[] PREFIX_ALLOWED_SHORT = {"dc"};

    /**
     * Allowed property prefixes for long version
     */
    // TODO: make this list configurable?
    protected static final String[] PREFIX_ALLOWED = {"dc", "cq", "crs", "xmp"};

    /**
     * SIREN properties prefix
     */
    public static final String PREFIX_SRN = "srn:";

    protected Logger log = LoggerFactory.getLogger(getClass());

    protected Resource resource;

    private Collection<Resource> children;

    /**
     * Relation attribute name "self".
     */
    public static final String REL_SELF = "self";

    /**
     * Relation attribute name "child".
     */
    public static final String REL_CHILD = "child";

    /**
     * Relation attribute name "content".
     */
    public static final String REL_CONTENT = "content";

    /**
     * Relation attribute name "next".
     */
    public static final String REL_NEXT = "next";

    /**
     * Relation attribute name "prev".
     */
    public static final String REL_PREV = "prev";

    /**
     * Relation attribute name "parent".
     */
    public static final String REL_PARENT = "parent";

    public AbstractSirenConverter(Resource resource) {
        this.resource = resource;
    }

    protected Collection<Resource> filterChildren(Map<String, String[]> filters, int offset, int limit) {
		return null;
	}

    /**
     * Returns the child resources.
     * @return A collection of child resources
     */
    protected Collection<Resource> listChildren() {
		return null;
	}

    /**
     * Checks a {@code key} against a set of {@code allowedPrefixes}.
     * @param key Key to check
     * @param allowedPrefixes Array of allowed prefixes
     * @return {@code true} if key is allowed, {@code false} otherwise
     */
    protected boolean isAllowedPrefix(String key, String[] allowedPrefixes) {
		return false;
	}

    /**
     * Checks a {@code key} against a set of {@code allowedPrefixes}.
     * @param key Key to check
     * @param context Converter context
     * @param allowedPrefixes Array of allowed prefixes
     * @return {@code true} if key is allowed, {@code false} otherwise
     */
    protected boolean isAllowedPrefix(String key, ResourceConverterContext context, String[] allowedPrefixes) {
		return false;
	}

    /**
     * Returns an array representation of the "class" property.
     * @return Siren class property values
     */
    protected abstract String[] getClazz() {
		return null;
	}

    /**
     * Returns an object representation of properties. By default all properties of the underlying resource matching
     * the {@link #PREFIX_ALLOWED_SHORT} or if {@code showAllProperties} is set to {@code true} the
     * {@link #PREFIX_ALLOWED} are returned.
     * @param context Converter context
     * @return Siren properties map
     */
    protected Map<String, Object> getProperties(ResourceConverterContext context) {
		return null;
	}

    /**
     * Returns an object representation of properties. By default all properties of the underlying resource matching
     * the {@link #PREFIX_ALLOWED_SHORT} or if {@code showAllProperties} is set to {@code true} the
     * {@link #PREFIX_ALLOWED} are returned.
     * @param context Converter context
     * @param isChild Indicator for child entity properties
     * @return Siren properties map
     */
    protected Map<String, Object> getProperties(ResourceConverterContext context, boolean isChild) {
		return null;
	}

    /**
     * Returns a set of properties after applying the property prefix rules to the specified {@code properties}.
     * @param properties Resource properties
     * @param context Converter context
     * @param isChild Indicator if resource is a child resource
     * @return
     */
    private Map<String, Object> getProperties(Map<String, Object> properties, ResourceConverterContext context,
        boolean isChild) {
			return null;
		}

    /**
     * Returns a list of entities. By default returns an empty list.
     * @param context ResourceConverterContext
     * @param children Children resources to get entities from
     * @return List of Siren sub-entities
     * @throws BuilderException If an error occurs during the build of the Entity
     */
    protected List<Entity> getEntities(ResourceConverterContext context, Iterator<Resource> children)
        throws BuilderException {
			return null;
		}

    /**
     * Returns a list of entities. By default returns an empty list.
     * @param context ResourceConverterContext
     * @return List of Siren sub-entities
     * @throws BuilderException If an error occurs during the build of the Entity
     */
    protected List<Entity> getEntities(ResourceConverterContext context) throws BuilderException {
		return null;
	}

    /**
     * Returns an entity object.
     * @param clazz Class attribute.
     * @param title Title attribute. Optional.
     * @param actions Actions collection. Optional.
     * @param entities Entities collection. Optional.
     * @param links Links collection. Optional.
     * @param properties Properties collection. Optional.
     * @return Siren main entity
     * @throws BuilderException If an error occurs during the build of the Entity
     */
    protected Entity getEntity(String[] clazz, String title, List<Action> actions, List<Entity> entities,
        List<Link> links, Map<String, Object> properties) throws BuilderException {
			return null;
		}

    /**
     * Returns a list of links. By default this method returns a list containing one link with rel attribute 'self'
     * and the href pointing to itself.
     * @param context Converter context
     * @return List of Siren links
     * @throws BuilderException is an error occurs during building the link
     * @throws ResourceConverterException if general error occurs
     */
    protected List<Link> getLinks(ResourceConverterContext context) throws BuilderException,
        ResourceConverterException {
			return null;
		}

    protected List<Link> getLinks(ResourceConverterContext context, boolean isChild) throws BuilderException,
        ResourceConverterException {
			return null;
		}

    /**
     * Returns a link representation.
     * @param rel rel attribute
     * @param href href attribute
     * @param type type attribute
     * @return A Siren link
     * @throws BuilderException If an error occurs during the build of the Link
     */
    protected Link getLink(String[] rel, String href, String type) throws BuilderException {
		return null;
	}

    /**
     * Returns a link representation.
     * @param rel rel attributes
     * @param href href attribute
     * @param type type attribute
     * @return A Siren link
     * @throws BuilderException If an error occurs during the build of the Link
     */
    protected Link getLink(String rel, String href, String type) throws BuilderException {
		return null;
	}

    /**
     * Returns a list of actions. By default this method returns an empty list.
     * @param context Converter context
     * @return A list of Siren actions
     * @throws BuilderException If an error occurs during the build of the Actions
     */
    protected List<Action> getActions(ResourceConverterContext context) throws BuilderException,
        ResourceConverterException {
			return null;
		}

    /**
     * {@inheritDoc}
     */
    @Override
    public Entity toEntity(ResourceConverterContext context) throws ResourceConverterException {
		return null;
	}

    @Override
    public Entity toSubEntity(ResourceConverterContext context) throws ResourceConverterException {
		return null;
	}

    /**
     * Builds an URL from the provided {@code resourcePath} and {@code extension} parameters.
     * @param context Converter context. Cannot be null.
     * @param resourcePath Resource path to build a URL from. Cannot be null.
     * @param extension Resource extension with leading dot or {@code null}
     * @return A string representation of an URL
     * @throws ResourceConverterException if a general error occurs
     */
    protected String buildURL(ResourceConverterContext context, String resourcePath, String extension)
        throws ResourceConverterException {
			return null;
		}

    /**
     * Builds an URL pointing to the 'next' results for paging.
     * @param context Converter context
     * @return A string representation of an URL or {@code null} if there is nothing more to show
     * @throws ResourceConverterException if an error occurs
     */
    protected String getNextPageURL(ResourceConverterContext context) throws ResourceConverterException {
		return null;
	}

    /**
     * Builds an URL pointing to the 'previous' results for paging.
     * @param context Converter context
     * @return A string representation of an URL
     * @throws ResourceConverterException if a general error occurs
     */
    protected String getPrevPageURL(ResourceConverterContext context) throws ResourceConverterException {
		return null;
	}

    private String buildPagingURL(ResourceConverterContext context, int offset) throws ResourceConverterException {
		return null;
	}

    private String buildURL(ResourceConverterContext context, String resourcePath, String extension,
        Map<String, String[]> additionalParameters) throws ResourceConverterException {
			return null;
		}
}
