/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.granite.ui.components;

import org.osgi.annotation.versioning.ProviderType;
import javax.servlet.http.HttpServletRequest;

/**
 * Utility class for form field component.
 */
@ProviderType
public class Field {
    /**
     * The config of the component
     */
    private Config cfg;

    /**
     * The name of the CSS class to apply in order to hide empty field
     * components in default mode.
     */
    public static String HIDE_IN_DEFAULT_CLASS = "foundation-field-hide-in-default";

    /**
     * The suffix that specifies if a property is a mixed value (for bulk editing).
     */
    public static String IS_MIXED_SUFFIX = ".granite.mixed";

    /**
     * The request attribute to be set when doing bulk editing
     */
    private static final String BULK_EDIT_MODE_ATTRIBUTE = "granite.ui.field.bulkedit";

    public Field() { /* Empty on purpose: don't use it! */ }

    /**
    * Creates a new Field object with the given config.
    *
    * @param cfg
    *            The config of the component
    */
    public Field(Config cfg) {
        this.cfg = cfg;
    }

    /**
     * Returns the CSS class (or space separated class values) that have to be
     * applied to the root element of field component. The provided class allows
     * for example to hide empty field components in read only mode.
     *
     * @param isEmpty
     *            Indicate if the field is empty
     * @return
     *            the CSS class (or space separated class values)
     */
    public String getRootClass(boolean isEmpty) {
		return null;
	}

    /**
     * Returns the CSS class (or space separated class values) that have to be
     * applied to the root element of field component. The provided class allows
     * for example to hide empty field components in read only mode.
     *
     * @param value
     *            The value of the field component
     * @return
     *            the CSS class (or space separated class values)
     */
    public String getRootClass(String value) {
		return null;
	}

    /**
     * Returns <code>true</code> if the field is a mixed value; <code>false</code> otherwise.
     *
     * A field is a mixed value if it has a property named <code>cfg.get("name") + IS_MIXED_SUFFIX</code> which has the value of <code>true</code>.
     *
     * @param value
     *            The form value
     * @return
     *            True if the field component holds a mixed value, false otherwise
     */
    public boolean isMixed(Value value) {
		return false;
	}

    /**
     * Returns <code>true</code> if bulk edit is allowed on this field; <code>false</code> otherwise.
     *
     * A field could be bulk edited if it has a property named <code>allowBulkEdit</code> which has the value of <code>true</code>
     *
     * @return
     *            True if the field component could be bulk edited, false otherwise
     */
    public boolean isBulkEditAllowed() {
		return false;
	}

    /**
     * Returns <code>true</code> if bulk edit mode is set; <code>false</code> otherwise
     *
     * Bulk edit mode is set if the request has an attribute <code>BULK_EDIT_MODE_ATTRIBUTE</code> which has the value of <code>true</code>
     *
     * @param request
     *            The request providing the parameter
     *
     * @return
     *            True if the bulk edit mode is set
     */
    public static boolean isBulkEditMode(HttpServletRequest request) {
		return false;
	}

    /**
     * Sets the bulk edit mode
     *
     * @param request
     *            The request that would hold the parameter
     */
    public static void setBulkEditMode(HttpServletRequest request) {
	}

    /**
     * Clears the bulk edit mode
     *
     * @param request
     *            The request that was holding the parameter
     */
    public static void clearBulkEditMode(HttpServletRequest request) {
	}

    /**
     * Returns the CSS class (or space separated class values) that have to be
     * applied to the root element of field component. The provided class allows
     * for example to hide empty field components in read only mode.
     *
     * @param cfg
     *            The config object of the field component
     * @param isEmpty
     *            Indicate if the field is empty
     * @return
     *            the CSS class (or space separated class values)
     */
    public static String getRootClass(Config cfg, boolean isEmpty) {
		return null;
	}

    /**
     * Returns the CSS class (or space separated class values) that have to be
     * applied to the root element of field component. The provided class allows
     * for example to hide empty field components in read only mode.
     *
     * @param cfg
     *            The config object of the field component
     * @param value
     *            The value of the field component
     * @return
     *            the CSS class (or space separated class values)
     */
    public static String getRootClass(Config cfg, String value) {
		return null;
	}

    /**
     * Returns <code>true</code> if the field is a mixed value; <code>false</code> otherwise.
     *
     * A field is a mixed value if it has a property named <code>cfg.get("name") + IS_MIXED_SUFFIX</code> which has the value of <code>true</code>.
     *
     * @param cfg
     *            The config object of the field component
     * @param value
     *            The form value
     * @return
     *            True if the field component holds a mixed value, false otherwise
     */
    public static boolean isMixed(Config cfg, Value value) {
		return false;
	}
}
