/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.granite.workflow;

import com.adobe.granite.workflow.collection.util.ResultSet;
import com.adobe.granite.workflow.exec.HistoryItem;
import com.adobe.granite.workflow.exec.InboxItem;
import com.adobe.granite.workflow.exec.Participant;
import com.adobe.granite.workflow.exec.Route;
import com.adobe.granite.workflow.exec.WorkItem;
import com.adobe.granite.workflow.exec.Workflow;
import com.adobe.granite.workflow.exec.WorkflowData;
import com.adobe.granite.workflow.exec.filter.InboxItemFilter;
import com.adobe.granite.workflow.exec.filter.WorkItemFilter;
import com.adobe.granite.workflow.model.VersionException;
import com.adobe.granite.workflow.model.WorkflowModel;
import com.adobe.granite.workflow.model.WorkflowModelFilter;
import com.adobe.granite.workflow.model.WorkflowNode;
import org.apache.sling.api.adapter.Adaptable;

import java.security.AccessControlException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * The <code>WorkflowSession</code> class provides all functionality (depending
 * on the users rights) for managing {@link WorkflowModel}s, {@link Workflow}
 * instances and their execution.
 * </p>
 * <p>
 * It provides methods to:
 * </p>
 * <ul>
 * <li>deploy/create a workflow models</li>
 * <li>start, stop, suspend, resume and complete a workflow instance</li>
 * <li>retrieve and complete {@link WorkItem}s assigned to a user/group</li>
 * </ul>
 */
public interface WorkflowSession extends Adaptable {
    /**
     * Deploys a new or modified <code>{@link WorkflowModel}</code>.
     *
     * @param model The <code>{@link WorkflowModel}</code> to be deployed.
     * @throws WorkflowException Thrown in case an error prevents deployment of the
     *                           <code>{@link WorkflowModel}</code>.
     */
    void deployModel(WorkflowModel model) throws WorkflowException {
	}

    /**
     * Creates a new blank <code>{@link WorkflowModel}</code>.
     *
     * @param title The title of the newly created
     *              <code>{@link WorkflowModel}</code>.
     * @return The newly created <code>{@link WorkflowModel}</code>.
     * @throws WorkflowException Thrown in case an error prevents creation of the
     *                           <code>{@link WorkflowModel}</code>.
     */
    WorkflowModel createNewModel(String title) throws WorkflowException {
		return null;
	}

    /**
     * Creates a new blank <code>{@link WorkflowModel}</code> under the given path.
     *
     * @param title The title of the newly created
     *              <code>{@link WorkflowModel}</code>.
     * @param id    The ID of the new <code>{@link WorkflowModel}</code>.
     * @return The newly created <code>{@link WorkflowModel}</code>.
     * @throws WorkflowException Thrown in case an error prevents creation of the
     *                           <code>{@link WorkflowModel}</code>.
     */
    WorkflowModel createNewModel(String title, String id) throws WorkflowException {
		return null;
	}

    /**
     * Deletes the given <code>{@link WorkflowModel}</code>.
     *
     * @param id The ID of the <code>{@link WorkflowModel}</code> to be
     *           deleted.
     * @throws WorkflowException Thrown in case an error prevents deployment of the
     *                           <code>{@link WorkflowModel}</code>.
     */
    void deleteModel(String id) throws WorkflowException {
	}

    /**
     * Returns newest versions of all deployed
     * <code>{@link WorkflowModel}</code>s.
     *
     * @return All deployed <code>{@link WorkflowModel}</code> at their newest
     *         versions.
     * @throws WorkflowException Thrown in case an error occurs while fetching the
     *                           <code>{@link WorkflowModel}</code>s.
     */
    WorkflowModel[] getModels() throws WorkflowException {
		return null;
	}

    /**
     * Returns newest versions of all deployed
     * <code>{@link WorkflowModel}</code>s.
     *
     * @param filter for workflow models
     * @return All deployed <code>{@link WorkflowModel}</code> at their newest
     *         versions.
     * @throws WorkflowException Thrown in case an error occurs while fetching the
     *                           <code>{@link WorkflowModel}</code>s.
     */
    WorkflowModel[] getModels(WorkflowModelFilter filter) throws WorkflowException {
		return null;
	}

    /**
     * Returns newest versions of all deployed
     * <code>{@link WorkflowModel}</code>s.
     *
     * @param start Start index of the result set.
     * @param limit Limit of the result set.
     * @return All deployed <code>{@link WorkflowModel}</code> at their newest
     *         versions.
     * @throws WorkflowException Thrown in case an error occurs while fetching the
     *                           <code>{@link WorkflowModel}</code>s.
     */
    ResultSet<WorkflowModel> getModels(long start, long limit) throws WorkflowException {
		return null;
	}

    /**
     * Returns newest versions of all deployed
     * <code>{@link WorkflowModel}</code>s.
     *
     * @param filter for workflow models
     * @param start Start index of the result set.
     * @param limit Limit of the result set.
     * @return All deployed <code>{@link WorkflowModel}</code> at their newest
     *         versions.
     * @throws WorkflowException Thrown in case an error occurs while fetching the
     *                           <code>{@link WorkflowModel}</code>s.
     */
    ResultSet<WorkflowModel> getModels(long start, long limit, WorkflowModelFilter filter) throws WorkflowException {
		return null;
	}

    /**
     * Returns newest version of the <code>{@link WorkflowModel}</code> with the
     * given ID.
     *
     * @param id The ID of the {@link WorkflowModel}.
     * @return The newest version of the <code>{@link WorkflowModel}</code> with
     *         the given ID or null, if no <code>{@link WorkflowModel}</code>
     *         with the given ID exists.
     * @throws WorkflowException Thrown in case an error occurs while fetching the
     *                           <code>{@link WorkflowModel}</code>.
     */
    WorkflowModel getModel(String id) throws WorkflowException {
		return null;
	}

    /**
     * Returns the <code>{@link WorkflowModel}</code> with the given ID and
     * version.
     *
     * @param id      The ID of the {@link WorkflowModel}.
     * @param version The version of the {@link WorkflowModel}.
     * @return The <code>{@link WorkflowModel}</code> with the given ID and
     *         version or null, if no <code>{@link WorkflowModel}</code> matches
     *         the given ID and/or version.
     * @throws WorkflowException Thrown in case an error occurs while fetching the
     *                           <code>{@link WorkflowModel}</code>.
     * @throws VersionException  Thrown in case the version does not exist or can not be
     *                           accessed.
     */
    WorkflowModel getModel(String id, String version) throws WorkflowException, VersionException {
		return null;
	}

    /**
     * Start a new {@link Workflow} instance with the given
     * <code>{@link WorkflowModel}</code> and <code>{@link WorkflowData}</code>.
     *
     * @param model <code>{@link WorkflowModel}</code> to be used for the new
     *              {@link Workflow} instance.
     * @param data  <code>{@link WorkflowData}</code> to be attached to the new
     *              workflow instance.
     * @return The <code>{@link Workflow}</code> of the newly started
     *         {@link Workflow} instance.
     * @throws WorkflowException Thrown in case the workflow process cannot be started.
     */
    Workflow startWorkflow(WorkflowModel model, WorkflowData data) throws WorkflowException {
		return null;
	}

    /**
     * Start a new {@link Workflow} instance with the given
     * <code>{@link WorkflowModel}</code>, <code>{@link WorkflowData}</code> and <code>meta data</code>.
     *
     * @param model    <code>{@link WorkflowModel}</code> to be used for the new
     *                 {@link Workflow} instance.
     * @param data     <code>{@link WorkflowData}</code> to be attached to the new
     *                 workflow instance.
     * @param metaData Meta data assigned to the new <code>{@link Workflow}</code>.
     * @return The <code>{@link Workflow}</code> of the newly started
     *         {@link Workflow} instance.          e
     * @throws WorkflowException Thrown in case the workflow process cannot be started.
     */
    Workflow startWorkflow(WorkflowModel model, WorkflowData data,
                           Map<String, Object> metaData) throws WorkflowException {
		return null;
	}

    /**
     * Terminates the given {@link Workflow} instance.
     *
     * @param instance The <code>{@link Workflow}</code> instance that shall be
     *                 terminated.
     * @throws WorkflowException Thrown in case the workflow instance cannot be terminated.
     */
    void terminateWorkflow(Workflow instance) throws WorkflowException {
	}

    /**
     * Resumes the given {@link Workflow} instance.
     *
     * @param instance The <code>{@link Workflow}</code> instance that shall be
     *                 resumed.
     * @throws WorkflowException Thrown in case the workflow instance cannot be resumed.
     */
    void resumeWorkflow(Workflow instance) throws WorkflowException {
	}

    /**
     * Suspends the given {@link Workflow} instance.
     *
     * @param instance The <code>{@link Workflow}</code> of the workflow instance
     *                 that shall be suspended.
     * @throws WorkflowException Thrown in case the workflow instance cannot be suspended.
     */
    void suspendWorkflow(Workflow instance) throws WorkflowException {
	}

    /**
     * All <code>{@link WorkItem}</code>s assigned to an active {@link Workflow}
     * instance and the respective user of the <code>WorkflowSession</code> will
     * be returned.
     *
     * @return All <code>{@link WorkItem}</code>s assigned to an active workflow
     *         and the respective users <code>WorkflowSession</code>.
     * @throws WorkflowException Thrown in case an exception occurs while fetching the active
     *                           <code>{@link WorkItem}</code>s.
     */
    WorkItem[] getActiveWorkItems() throws WorkflowException {
		return null;
	}

    /**
     * All <code>{@link WorkItem}</code>s assigned to an active {@link Workflow}
     * instance and the respective user of the <code>WorkflowSession</code> will
     * be returned.
     *
     * @param start Start index of the result set.
     * @param limit Limit of the result set.
     * @return All <code>{@link WorkItem}</code>s assigned to an active workflow
     *         and the respective users <code>WorkflowSession</code>.
     * @throws WorkflowException Thrown in case an exception occurs while fetching the active
     *                           <code>{@link WorkItem}</code>s.
     */
    ResultSet<WorkItem> getActiveWorkItems(long start, long limit) throws WorkflowException {
		return null;
	}

    /**
     * All <code>{@link WorkItem}</code>s assigned to an active {@link Workflow}
     * instance and the respective user of the <code>WorkflowSession</code> will
     * be returned.
     *
     * @param start  Start index of the result set.
     * @param limit  Limit of the result set.
     * @param filter filter for work items
     * @return All <code>{@link WorkItem}</code>s assigned to an active workflow
     *         and the respective users <code>WorkflowSession</code>.
     * @throws WorkflowException Thrown in case an exception occurs while fetching the active
     *                           <code>{@link WorkItem}</code>s.
     */
    ResultSet<WorkItem> getActiveWorkItems(long start, long limit, WorkItemFilter filter) throws WorkflowException {
		return null;
	}

    /**
     * Retrieves all the active {@link InboxItem}s assigned to user for this {@link WorkflowSession} instance.
     *
     * Use this method instead of {@link com.adobe.granite.workflow.WorkflowSession#getActiveWorkItems(long, long, com.adobe.granite.workflow.exec.filter.WorkItemFilter)} ()}
     * if you wish to retrieve both WorkItems (which is an InboxItem) and InboxItems from other InboxItem providers
     * @param start  Start index of the result set.  zero based, pass 0 to start at beginning
     * @param limit  Limit of the result set.  pass -1 for no limit.
     * @param filter filter for inbox items
     * @return a ResultSet of Inbox Items
     * @throws WorkflowException Thrown in case an exception occurs while fetching all <code>{@link InboxItem}</code>s.
     */
    ResultSet<InboxItem> getActiveInboxItems(long start, long limit, InboxItemFilter filter) throws WorkflowException {
		return null;
	}

    /**
     * Retrieves all the active {@link InboxItem}s assigned to user for this {@link WorkflowSession} instance.
     *
     * Use this method instead of {@link com.adobe.granite.workflow.WorkflowSession#getActiveWorkItems(long, long, com.adobe.granite.workflow.exec.filter.WorkItemFilter)} ()}
     * if you wish to retrieve both WorkItems (which is an InboxItem) and InboxItems from other InboxItem providers
     * @param start  Start index of the result set.  zero based, pass 0 to start at beginning
     * @param limit  Limit of the result set.  pass -1 for no limit.
     * @param itemSubType The sub-type of the inbox item.
     * @param filter filter for inbox items
     * @return a ResultSet of Inbox Items
     * @throws WorkflowException Thrown in case an exception occurs while fetching all <code>{@link InboxItem}</code>s.
     */
    ResultSet<InboxItem> getActiveInboxItems(long start, long limit, String itemSubType, InboxItemFilter filter) throws WorkflowException {
		return null;
	}

    /**
     * All <code>{@link WorkItem}</code>s assigned to a {@link Workflow}
     * instance and assigned to the respective user of the
     * <code>WorkflowSession</code> will be returned.
     *
     * @return All <code>{@link WorkItem}</code>s assigned to a {@link Workflow}
     *         instance and the respective users <code>WorkflowSession</code>.
     * @throws WorkflowException Thrown in case an exception occurs while fetching all
     *                           <code>{@link WorkItem}</code>s.
     */
    WorkItem[] getAllWorkItems() throws WorkflowException {
		return null;
	}


    /**
     * All <code>{@link WorkItem}</code>s assigned to a {@link Workflow}
     * instance and assigned to the respective user of the
     * <code>WorkflowSession</code> will be returned.
     *
     * @param start Start index of the result set.
     * @param limit Limit of the result set.
     * @return All <code>{@link WorkItem}</code>s assigned to a {@link Workflow}
     *         instance and the respective users <code>WorkflowSession</code>.
     * @throws WorkflowException Thrown in case an exception occurs while fetching all
     *                           <code>{@link WorkItem}</code>s.
     */
    ResultSet<WorkItem> getAllWorkItems(long start, long limit) throws WorkflowException {
		return null;
	}

    /**
     * Returns the {@link WorkItem} with the given ID assigned to a
     * {@link Workflow} instance, but only if the respective users
     * {@link WorkflowSession} has access to it.
     *
     * @param id The ID of the {@link WorkItem}.
     * @return The {@link WorkItem} with the given ID.
     * @throws WorkflowException Thrown in case an exception is occurs while fetching the
     *                           {@link WorkItem}.
     */
    WorkItem getWorkItem(String id) throws WorkflowException {
		return null;
	}

    /**
     * Returns all {@link Workflow} instances that are in one of
     * the given states and to which the
     * {@link WorkflowSession} has access to.
     *
     * @param states The list of states used to select the {@link Workflow} instances.
     * @return All {@link Workflow}s that are in one of the given
     *         states and to which the {@link WorkflowSession} has access to.
     * @throws WorkflowException Thrown in case an error occurred while retrieving the
     *                           {@link Workflow}s.
     */
    Workflow[] getWorkflows(String[] states) throws WorkflowException {
		return null;
	}

    /**
     * Returns all {@link Workflow}s instances that are in one of
     * the given states and to which the {@link WorkflowSession} has access to.
     *
     * @param states The list of states used to select the {@link Workflow} instances.
     * @param start  Start index of the result set.
     * @param limit  Limit of the result set.
     * @return All {@link Workflow}s that are in one of the given
     *         states and to which the {@link WorkflowSession} has access to.
     * @throws WorkflowException Thrown in case an error occurred while retrieving the
     *                           {@link Workflow}s.
     */
    ResultSet<Workflow> getWorkflows(String[] states, long start, long limit) throws WorkflowException {
		return null;
	}

    /**
     * Returns all {@link Workflow}s which the
     * <code>{@link WorkflowSession}</code> has access to.
     *
     * @return All <code>{@link Workflow}</code>s the
     *         <code>{@link WorkflowSession}</code> has access to.
     * @throws WorkflowException Thrown in case an error occurred while retrieving the
     *                           <code>{@link Workflow}</code>s.
     */
    Workflow[] getAllWorkflows() throws WorkflowException {
		return null;
	}

    /**
     * Retrieve the <code>{@link Workflow}</code> instance with the given ID.
     *
     * @param id The ID of the <code>{@link Workflow}</code> instance that
     *           should be returned.
     * @return The <code>{@link Workflow}</code> instance with the given ID or
     *         null, if no <code>{@link Workflow}</code> instance ID matches the
     *         given ID.
     * @throws WorkflowException Thrown in case an unexpected error occurs while retrieving
     *                           the <code>{@link Workflow}</code>.
     */
    Workflow getWorkflow(String id) throws WorkflowException {
		return null;
	}

    /**
     * Completes the <code>{@link WorkItem}</code> for the assigned
     * <code>{@link Workflow}</code> instance and assign to the <i>next</i>
     * <code>{@link WorkflowNode}</code> according to the given
     * <code>{@link Route}</code>.
     *
     * @param item  The <code>{@link WorkItem}</code> to be completed.
     * @param route The <code>{@link Route}</code> to the destination to be
     *              advanced.
     * @throws WorkflowException Thrown in case an error is occurs while completing the
     *                           <code>{@link WorkItem}</code>.
     */
    void complete(WorkItem item, Route route) throws WorkflowException {
	}

    /**
     * All <code>{@link Route}</code> to all possible destinations will be
     * returned. Rules will be already executed, thus all routes that are active
     * can be selected.
     *
     * @param item   The <code>{@link WorkItem}</code> to retrieve routes from.
     * @param expand Expands the group members if the destination points to a
     *               group. Means that not only the group is returned as route but
     *               as well all group members.
     * @return All <code>{@link Route}</code>s to advance.
     * @throws WorkflowException Thrown in case an error is occurs while evaluating the
     *                           <code>{@link Route}</code>s.
     */
    List<Route> getRoutes(WorkItem item, boolean expand) throws WorkflowException {
		return null;
	}

    /**
     * All <code>{@link Route}</code> to all possible already passed
     * destinations will be returned. Rules will be already executed, thus all
     * routes that are active can be selected.
     *
     * @param item   The <code>{@link WorkItem}</code> to retrieve back routes
     *               from.
     * @param expand Expands the group members if the destination points to a
     *               group. Means that not only the group is returned as route but
     *               as well all group members.
     * @return All <code>{@link Route}</code>s to advance (back).
     * @throws WorkflowException Thrown in case an error is occurs while evaluating the
     *                           <code>{@link Route}</code>s.
     */
    List<Route> getBackRoutes(WorkItem item, boolean expand) throws WorkflowException {
		return null;
	}

    /**
     * Creates a new <code>{@link WorkflowData}</code> instance based on the
     * given data.
     *
     * @param payloadType The type of payload for the new
     *                    <code>{@link WorkflowData}</code> instance.
     * @param payload     The payload object used for creating the new
     *                    <code>{@link WorkflowData}</code> instance.
     *                    <code>{@link WorkflowData}</code> instance.
     * @return The new <code>{@link WorkflowData}</code> instance.
     */
    WorkflowData newWorkflowData(String payloadType, Object payload) {
		return null;
	}

    /**
     * Returns all <code>delegates</code> as <code>Participant</code> who are
     * allowed to take over the <code>WorkItem</code>
     *
     * @param item The workitem
     * @return The list of <code>Participant</code>s who are allowed to act as delegates.
     * @throws WorkflowException in case the delegates could not be retrieved for some reason
     */
    Iterator<Participant> getDelegates(WorkItem item) throws WorkflowException {
		return null;
	}

    /**
     * Delegates the <code>WorkItem</code> to the specified <code>Principal</code>.
     *
     * @param item        the corresponding workitem
     * @param participant the participant to delegate
     * @throws java.security.AccessControlException
     *                           if the given participant is not allowed to take over the workitem.
     * @throws WorkflowException in case the delegation fails in regards of persisting the "new" state.
     */
    void delegateWorkItem(WorkItem item, Participant participant) throws WorkflowException, AccessControlException {
	}

    /**
     * The complete ordered list of {@link com.adobe.granite.workflow.exec.HistoryItem}
     * s is returned for the given {@link Workflow} instance.
     *
     * @param instance {@link Workflow} instance
     * @return <code>List</code> of {@link com.adobe.granite.workflow.exec.HistoryItem}
     *         s
     * @throws WorkflowException in case the history item retrieval fails
     */
    List<HistoryItem> getHistory(Workflow instance) throws WorkflowException {
		return null;
	}

    /**
     * Updates the workflow data of a running workflow instance to the given
     * <code>WorkflowData</code>.
     *
     * @param instance The {@link Workflow} instance for which to update the data.
     * @param data     The {@link WorkflowData} for the update.
     */
    void updateWorkflowData(Workflow instance, WorkflowData data) {
	}

    /**
     * The logout method has to be called in order to clean up all jcr session
     * references.
     */
    void logout() {
	}

    /**
     * Checks for the workflow superuser
     *
     * @return {@code true} if the user is a workflow superuser
     */
    boolean isSuperuser() {
		return false;
	}

    /**
     * <p>
     * This method restarts a workflow instance that is in a "stale" state.
     * means there is no workitem around. this could happen in case the eventing
     * which controls the the script/java processes purges an event/job (there
     * is the depending (serialized) workitem stored). The restart tries to
     * start the workflow at the position right before the workitem got lost.
     * </p><p>
     * <b>Note</b>: This is an administrative method and can only be used by a
     * workflow superuser!
     * </p>
     * @param workflow workflow
     * @throws WorkflowException in case an error occurs
     */
    void restartWorkflow(Workflow workflow) throws WorkflowException {
	}
}
