// =================================================================================================
// ADOBE SYSTEMS INCORPORATED
// Copyright 2012 Adobe Systems Incorporated
// All Rights Reserved
//
// NOTICE:  Adobe permits you to use, modify, and distribute this file in accordance with the terms
// of the Adobe license agreement accompanying it.
// =================================================================================================
package com.adobe.xmp.path;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Map.Entry;

/**
 * <code>XMPPath</code> is a Class that provides an easy iterative description of a specific path into the XMP tree.
 * Each {@link XMPPathSegment} represents one segment of the path into the XMP tree.  
 */
public class XMPPath implements List<XMPPathSegment> 
{
	private List<XMPPathSegment> segments;

	public enum Compare
	{
		DIFFERENT, EQUAL, ANCESTOR, DESCENDANT
	}
	
	/**
	 * Creates an empty <code>XMPPath</code> object
	 */
	public XMPPath() 
	{
		// ArrayList is supposed to be faster than Vector, especially for iterations 
		segments = new ArrayList<XMPPathSegment>();
	}
	
	/**
	 * Creates an <code>XMPPath</code> object by parsing an XMP path String and creates one path segment 
	 * for each detected segment in the path.
	 * @param path Path string to parse
	 * @param prefixContract A Map that contains the mapping between Namespaces and their prefixes.
	 * This must be provided to interpret the path correctly. The key must be the prefix and the value the namespace!
	 * @throws XMPPathParserException If the path has an invalid format or a prefix is found that is not defined in the prefixContract
	 */
	public static XMPPath parse(String path, Map<String, String> prefixContract ) throws XMPPathParserException
	{
		return null;
	}
	
	/**
	 * Serializes the <code>XMPPath</code> object to a String representation.
	 * This will produce a long form of the path using the full namespace strings.
	 * @return String representation of the <code>XMPPath</code>
	 */
	public String serialize()
	{
		return null;
	}
	
	/**
	 * Serializes the <code>XMPPath</code> object to a String representation.
	 * This produces a short form of the path using the prefixes provided by the contract.
	 * @param prefixContract A Map that contains the mapping between Namespaces and their prefixes.
	 * The key must be the prefix and the value the namespace!
	 * @return String representation of the <code>XMPPath</code>
	 */
	public String serialize( Map<String, String> prefixContract )
	{
		return null;
	}
	
	
	public int size()
	{
		return 0;
	}

	public boolean isEmpty()
	{
		return false;
	}

	public boolean contains(Object o)
	{
		return false;
	}

	public Iterator<XMPPathSegment> iterator()
	{
		return null;
	}

	public Object[] toArray()
	{
		return null;
	}

	public <T> T[] toArray(T[] a)
	{
		return null;
	}

	public boolean add(XMPPathSegment e)
	{
		return false;
	}

	public boolean remove(Object o)
	{
		return false;
	}

	public boolean containsAll(Collection<?> c)
	{
		return false;
	}

	public boolean addAll(Collection<? extends XMPPathSegment> c)
	{
		return false;
	}

	public boolean addAll(int index, Collection<? extends XMPPathSegment> c)
	{
		return false;
	}

	public boolean removeAll(Collection<?> c)
	{
		return false;
	}

	public boolean retainAll(Collection<?> c)
	{
		return false;
	}

	public void clear()
	{
	}

	public XMPPathSegment get(int index)
	{
		return null;
	}

	public XMPPathSegment set(int index, XMPPathSegment element)
	{
		return null;
	}

	public void add(int index, XMPPathSegment element)
	{
	}

	public XMPPathSegment remove(int index)
	{
		return null;
	}

	public int indexOf(Object o)
	{
		return 0;
	}

	public int lastIndexOf(Object o)
	{
		return 0;
	}

	public ListIterator<XMPPathSegment> listIterator()
	{
		return null;
	}

	public ListIterator<XMPPathSegment> listIterator(int index)
	{
		return null;
	}

	public List<XMPPathSegment> subList(int fromIndex, int toIndex)
	{
		return null;
	}

	/**
	 * Compares the provided path with this path.
	 * 
	 * @param path the path to compare with
	 * @return The paths are equal if both have the same size and each segment is EQUAL.
	 * If the provided path is a subpath of this path, ANCESTOR is returned.
	 * If the provided path is an ancestor of this path, DESCENDANT is returned.
	 * In all other cases the paths are DIFFERENT.
	 * @throws IllegalArgumentException If path is null
	 */
	public Compare compare(XMPPath path) 
	{
		return null;
	}

	@Override
	public int hashCode() 
	{
		return 0;
	}

	@Override
	public boolean equals(Object obj) 
	{
		return false;
	}
}
