/*
 * File: PropertyInfo.java
 * ************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2011 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.xmp.schema.rng.model;

import java.util.HashMap;
import java.util.Map;
import java.util.Stack;

import javax.xml.namespace.QName;

import com.adobe.xmp.schema.model.ArrayType.ArrayForm;
import com.adobe.xmp.schema.rng.model.ParamInfoGroup.Operator;
import com.adobe.xmp.schema.rng.parser.annotation.RNGDecorationAnnotation.ACCESS;
import com.adobe.xmp.schema.rng.parser.annotation.RNGDecorationAnnotation.CHOICE;

/**
 * This class holds relevant data for constructing a {@link com.adobe.xmp.schema.model.PropertyDescription} object collected from parsing the
 * RelaxNG schema.
 * 
 * @author pwollek
 */
public class PropertyInfo implements PropertyAnnotationInfo
{
	private String mNS;

	private String mName;

	private ParamInfoGroup mParamInfos;

	private DatatypeInfo mRawDataType;

	private DatatypeInfo mDataType;

	private Stack<PropertyInfo> mFields; // struct fields

	private Stack<PropertyInfo> mQualifiers; // qualifiers

	private ArrayForm mArrayForm;

	private String mLabel;

	private String mDescription;

	private ACCESS mAccess;

	private CHOICE choice;

	private boolean mDeprecated;

	private boolean mMandatory;

	private boolean mIsQualifier;
	
	private boolean mIsArrayItemQualifier;

	private SchemaInfo mSchemaInfo;

	/**
	 * Store custom decorators with arbitrary QNames, that contain a property-value list.
	 * The outer Map maps the QName to a second property-value Map, that contains the actual decorator properties.
	 */
	private HashMap<QName, Map<String, String>> mDecoratorStore;
	
	/**
	 * 
	 * Constructs a new PropertyInfo.
	 * 
	 * @param ns
	 *            Namespace of the property
	 * @param name
	 *            Name of the property
	 * @param schemaInfo
	 *            {@link SchemaInfo} object, which stores information retrieved from annotations about the XMP schema
	 */
	public PropertyInfo(String ns, String name, SchemaInfo schemaInfo)
	{
		mNS = ns;
		mName = name;
		mParamInfos = new ParamInfoGroup(Operator.kAND);
		// Raw data type by default is set to text
		mRawDataType = new DatatypeInfo("Text", true);
		mFields = new Stack<PropertyInfo>();
		mQualifiers = new Stack<PropertyInfo>();
		mArrayForm = null;
		mLabel = "";
		mDescription = "";
		mAccess = null;
		mDeprecated = false;
		mMandatory = true;
		mIsQualifier = false;
		mIsArrayItemQualifier = false;
		mSchemaInfo = schemaInfo;
	}

	/**
	 * Adds a child, which could either be a property qualifier, array item qualifier or a struct field
	 * 
	 * @param child
	 *            child to add
	 */
	public void addChild(PropertyInfo child)
	{
	}

	/**
	 * @return List of qualifiers of this {@link PropertyInfo}
	 */
	public Stack<PropertyInfo> getQualifiers()
	{
		return null;
	}

	/**
	 * @return List of fields of this {@link PropertyInfo}. Relevant only for struct property.
	 */
	public Stack<PropertyInfo> getFields()
	{
		return null;
	}

	/**
	 * @param mNS
	 *            the mNS to set
	 */
	public void setNS(String mNS)
	{
	}

	/**
	 * @return the mNS
	 */
	public String getNS()
	{
		return null;
	}

	/**
	 * @param mName
	 *            the mName to set
	 */
	public void setName(String mName)
	{
	}

	/**
	 * @return the mName
	 */
	public String getName()
	{
		return null;
	}

	/**
	 * @param mParamInfos
	 *            the mParamInfos to set
	 */
	public void setParamInfos(ParamInfoGroup mParamInfos)
	{
	}

	/**
	 * @return the mParamInfos
	 */
	public ParamInfoGroup getParamInfos()
	{
		return null;
	}

	/**
	 * @param mRawDataType
	 *            the mRawDataType to set
	 */
	public void setRawDataType(DatatypeInfo mRawDataType)
	{
	}

	/**
	 * @return the mRawDataType
	 */
	public DatatypeInfo getRawDataType()
	{
		return null;
	}

	/**
	 * @param mDataType
	 *            the mDataType to set
	 */
	public void setDataType(DatatypeInfo mDataType)
	{
	}

	/**
	 * @return the mDataType
	 */
	public DatatypeInfo getDataType()
	{
		return null;
	}

	/**
	 * @param form
	 *            the mArrayForm to set
	 */
	public void setArrayForm(ArrayForm form)
	{
	}

	/**
	 * @return the mArrayForm
	 */
	public ArrayForm getArrayForm()
	{
		return null;
	}

	/**
	 * @param mLabel
	 *            the mLabel to set
	 */
	public void setLabel(String mLabel)
	{
	}

	/**
	 * @return the mLabel
	 */
	public String getLabel()
	{
		return null;
	}

	/**
	 * @param mDescription
	 *            the mDescription to set
	 */
	public void setDescription(String mDescription)
	{
	}

	/**
	 * @return the mDescription
	 */
	public String getDescription()
	{
		return null;
	}

	/**
	 * @param mAccess
	 *            the mAccess to set
	 */
	public void setAccess(ACCESS mAccess)
	{
	}

	/**
	 * @return the mAccess
	 */
	public ACCESS getAccess()
	{
		return null;
	}

	/**
	 * @param mDeprecated
	 *            the mDeprecated to set
	 */
	public void setDeprecated(boolean mDeprecated)
	{
	}

	/**
	 * @return the mDeprecated
	 */
	public boolean isDeprecated()
	{
		return false;
	}

	/**
	 * @param mMandatory
	 *            the mMandatory to set
	 */
	public void setMandatory(boolean mMandatory)
	{
	}

	/**
	 * @return the mMandatory
	 */
	public boolean isMandatory()
	{
		return false;
	}
	
	/**
	 * Sets if this propertyInfo stores information for an array item qualifier.
	 * @param isArrayItemQualifier boolean
	 */
	public void setIsArrayItemQualifier(boolean isArrayItemQualifier)
	{
	}
	
	/**
	 * Gets if the propertyInfo holds information for an array item qualifier.
	 * @return boolean
	 */
	public boolean isArrayItemQualifier()
	{
		return false;
	}
	

	/**
	 * @param isQualifier
	 *            the mIsQualifier to set
	 */
	public void setIsQualifier(boolean isQualifier)
	{
	}

	/**
	 * @return the mIsQualifier
	 */
	public boolean isQualifier()
	{
		return false;
	}

	/**
	 * @param mSchemaInfo
	 *            the mSchemaInfo to set
	 */
	public void setSchemaInfo(SchemaInfo mSchemaInfo)
	{
	}

	/**
	 * @return the mSchemaInfo
	 */
	public SchemaInfo getSchemaInfo()
	{
		return null;
	}

	/**
	 * @param choice
	 *            the choice to set
	 */
	public void setChoice(CHOICE choice)
	{
	}

	/**
	 * @return the choice
	 */
	public CHOICE getChoice()
	{
		return null;
	}
	
	/**
	 * Returns if this property has decorators
	 * @return boolean
	 */
	public boolean hasDecorators()
	{
		return false;
	}
	
	/**
	 * Gets the decorator store for this {@link PropertyInfo}
	 * @return HashMap
	 */
	public HashMap<QName, Map<String, String>> getDecoratorStore()
	{
		return null;
	}
	
	/**
	 * Gets decorator map for given {@link QName}. If entry does not 
	 * exist for the given qname, an entry is created in the decoratorstore.
	 * @param qname qname
	 * @return Map
	 */
	public Map<String, String> getDecorator(QName qname)
	{
		return null;
	}

	@Override
	public String toString()
	{
		return null;
	}
	
}
