/*******************************************************************************
 * ADOBE CONFIDENTIAL
 * __________________
 *
 * Copyright 2017 Adobe Systems Incorporated
 * All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 ******************************************************************************/

package com.day.cq.analytics.testandtarget.mac;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.HashMap;
import java.util.Map;

import aQute.bnd.annotation.ProviderType;
import org.apache.commons.lang.StringUtils;

import com.day.cq.analytics.testandtarget.TargetConstants;

/**
 * Holds the Marketing Cloud Metadata which is attached to objects exported to Target. Not all objects require this
 * metadata, so for now it's only attached to offers and activities
 * 
 */
@ProviderType
public class MacMetadata {

    private final Map<String, Object> metadata = new HashMap<String, Object>();

    public MacMetadata() {

    }

    /**
     * Adds a Marketing Cloud Metadata attribute / value pair to this offerr request. The following restrictions apply:
     * <ul>
     * <li>the attributeName or attributeValue must not exceed 250 chars in length</li>
     * <li>The maximum size of the MAC Metadata list is 15 items</li>
     * </ul>
     * This method throws a {@link RuntimeException} if any of the above mentioned restrictions are hit.
     *
     * @param attributeName  the name of the attribute
     * @param attributeValue the value of the attribute
     */
    @Nonnull
    public MacMetadata addAttribute(@Nonnull String attributeName, @Nonnull String attributeValue) {
		return null;
	}

    public MacMetadata addAttribute(@Nonnull String attributeName, @Nonnull String[] attributeValues) {
		return null;
	}

    /**
     * Removes a Marketing Cloud Metadata attribute from the list.
     *
     * @param attributeName the name of the attribute.
     */
    public void removeAttribute(@Nonnull String attributeName) {
	}

    /**
     * Retrieves the list of the Marketing Cloud Metadata attributes.
     *
     * @return a {@link Map} having the attribute names as keys and the attribute values as values.
     */
    public Map<String, String> getMarketingCloudMetadata() {
		return null;
	}


    /**
     * Retrieves the value of a specific attribute from this metadata set
     * @param attributeName the name of the attribute
     * @return a {@link String} containing the value, or {@code null} if the value is not found or it's not a {@link String}.
     */
    @Nullable
    public String getValue(@Nonnull String attributeName) {
		return null;
	}

    /**
     * Retrieves the value of a specific attribute from this metadata set, specifying what type should the returned value be
     * @param attributeName the name of the attribute
     * @return the value cast to the specified type
     */
    @Nullable
    public String[] getValues(@Nonnull String attributeName) {
		return null;
	}

    /**
     * Generates the mandatory metadata. The mandatory metadata contains the following attributes:
     * <ul>
     * <li>The edit url of the object, i.e. the full URL used to open the object in it's editor in AEM</li>
     * <li>The user that perform the last modification</li>
     * <li>The source product name (in our case Adobe Experience Manager)</li>
     * </ul>
     *
     * @param editUrl          the URL which is used to access the object
     * @param remoteModifiedBy the user which performed the last modification
     * @return a {@link MacMetadata} object containing the three attributes above
     */
    @Nonnull
    public static MacMetadata generateMandatoryMetadata(@Nonnull String editUrl, @Nonnull String remoteModifiedBy) {
		return null;
	}
}
