/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.commons.servlets;

import org.apache.commons.collections.Predicate;
import org.apache.felix.scr.annotations.sling.SlingServlet;
import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.SlingHttpServletResponse;
import org.apache.sling.api.request.RequestPathInfo;
import org.apache.sling.api.resource.Resource;

import javax.servlet.ServletException;
import java.io.IOException;

/**
 * Forwards to the resource considering the resource search path ("/apps", "/libs", etc.).
 * If a suffix is provided a resource of the name of the requested resource will be searched
 * inside the paths provided by the suffix. If the suffix contains a period after the last
 * slash the part behind this period will be ignored.<br>
 * <br>
 * Sample without suffix:<br>
 * Request: /libs/x/y/z.overlay.infinity.json<br>
 * The request will be forwarded to the first resource found in the following order
 * (taking "/apps" and "/libs" as given resource search paths):<br>
 * /apps/x/y/z.infinity.json<br>
 * /libs/x/y/z.infinity.json<br>
 * <br>
 * Sample with suffix:<br>
 * Request: /libs/x/y/z.overlay.infinity.json/a/b.json<br>
 * The request will be forwarded to the first resource found in the following order
 * (taking "/apps" and "/libs" as given resource search paths):<br>
 * /apps/x/y/a/b/z.infinity.json<br>
 * /libs/x/y/a/b/z.infinity.json<br>
 * /apps/x/y/a/z.infinity.json<br>
 * /libs/x/y/a/z.infinity.json<br>
 * /apps/x/y/z.infinity.json<br>
 * /libs/x/y/z.infinity.json<br>
 */
@SlingServlet(
        selectors = OverlayServlet.OVERLAY,
        resourceTypes = "sling/servlet/default",
        methods = "GET",
        metatype = false
)
public class OverlayServlet extends AbstractPredicateServlet {

    private static final long serialVersionUID = 2266516903915391606L;

    /**
     * selector value
     */
    public static final String OVERLAY = "overlay";

    /**
     * {@inheritDoc}
     */
    @Override
    protected void doGet(SlingHttpServletRequest req,
                         SlingHttpServletResponse resp,
                         Predicate predicate)
            throws ServletException, IOException {
			}

}