/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.dam.commons.util;

import java.io.IOException;
import java.util.Map;

import javax.imageio.IIOException;

import com.day.cq.dam.api.Asset;
import static com.day.cq.dam.api.DamConstants.EXIF_PIXELXDIMENSION;
import static com.day.cq.dam.api.DamConstants.EXIF_PIXELYDIMENSION;
import static com.day.cq.dam.api.DamConstants.TIFF_IMAGELENGTH;
import static com.day.cq.dam.api.DamConstants.TIFF_IMAGEWIDTH;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This class offers some useful memory management functions for asset/image processing
 */
public class MemoryUtil {

    private static final String TIFF_BITS_PER_SAMPLE = "tiff:BitsPerSample";

	private static final String DAM_BITSPERPIXEL = "dam:Bitsperpixel";

	private static final String PHOTOSHOP_COLOR_MODE = "photoshop:ColorMode";

	private static final String TIFF_SAMPLES_PER_PIXEL = "tiff:SamplesPerPixel";

	/**
     * Logger instance for this class.
     */
    private static final Logger log = LoggerFactory
            .getLogger(MemoryUtil.class);

    /**
     * We assume about 100 megs as minimal cq requirement...
     */
    public static long MIN_CQ_MEMORY_REQUIREMENT = 1024 * 1024 * 100;

    private static final int DEFAULT_MAX_TRIALS = 100;

    /**
     * Callback throwing IOException and returning some (image) object
     * for {@link #tryUntilEnoughMemory(com.day.cq.dam.api.Asset, int, com.day.cq.dam.commons.util.MemoryUtil.Callback)}
     */
    public interface Callback<T> {
        T execute() throws IOException {
			return null;
		}
    }

    /**
     * Tries to execute the callback until no memory exception occurs anymore. Will first check
     * if the asset can be loaded with the given JVM memory settings at all, then go into a
     * loop catching "not enough memory" exceptions and retry until loading worked or a maximum
     * number of retries is reached (100). If it fails, <code>null</code> will be returned.
     *
     * @param asset asset to load Java 2D image from
     * @param callback method that actually loads an image involving Java 2D memory,
     *                 return value will be passed through and returned by this method
     * @param <T> return type (image object etc.)
     * @return the value returned by the callback if successful, or <code>null</code>
     *         if memory limits prevented loading within a certain number of trials
     */
    public static <T> T tryUntilEnoughMemory(Asset asset, Callback<T> callback) {
		return null;
	}

    /**
     * Tries to execute the callback until no memory exception occurs anymore. Will first check
     * if the asset can be loaded with the given JVM memory settings at all, then go into a
     * loop catching "not enough memory" exceptions and retry until loading worked or the given
     * maximum number of retries is reached. If it fails, <code>null</code> will be returned.
     *
     * @param asset asset to load Java 2D image from
     * @param maxTrials maximum number of trials, must be &gt; 0
     * @param callback method that actually loads an image involving Java 2D memory,
     *                 return value will be passed through and returned by this method
     * @param <T> return type (image object etc.)
     * @return the value returned by the callback if successful, or <code>null</code>
     *         if memory limits prevented loading within a certain number of trials
     */
    public static <T> T tryUntilEnoughMemory(Asset asset, final int maxTrials, Callback<T> callback) {
		return null;
	}

    /**
     * Checks if the available max. memory is enough to process the image
     *
     * @param asset asset to check
     * @return <code>true</code> either if the check succeeds or the check cannot
     *         be performed (no image data available e.g. the asset is not an image)
     *         otherwise <code>false</code>
     */
    public static boolean hasEnoughSystemMemory(Asset asset) {
		return false;
	}

    /**
     * Calculates the expected image memory consumption. Bytes required by each pixel are calculated from
     * metadata fields like <code>photoshop:ColorMode</code> or <code>tiff:SamplesPerPixel</code>.
     * If this information can be derived from metadata then it is defaulted to 4.
     *
     * @param asset asset to check
     * @return the calculated amount or <code>-1</code> in case the expected
     * memory cannot be calculated
     */
    public static long getExpectedImageMemory(Asset asset) {
		return 0L;
	}
    
	/**
	 * Find pixel size from the asset metadata if available. If it can not
	 * calculate from metadata then returns 4.
	 *
	 * @return pixel size in bytes
	 */
	private static int getPixelSize(Asset asset, Map<String, Object> metadata) {
		return 0;
	}
    
	/**
	 * gets number of bits required per channel or per sample in a pixel. Pixel
	 * have different channels like R G B or C M Y K
	 */
	private static long getBitsPerSample(Asset asset,
			Map<String, Object> metadata) {
				return 0L;
			}

	private static long getAsLong(Asset asset, Map<String, Object> metadata,
			String name) {
				return 0L;
			}

	private static long getAsLong(Asset asset, Object value) {
		return 0L;
	}

    /**
     * Checks if enough memory is available for the asset/image processing
     * Note: it works for images only!
     *
     * @param asset asset to check
     * @return <code>true</code> either if the check succeeds or the check cannot
     *         be performed (no image data available e.g. the asset is not an image)
     *         otherwise <code>false</code>
     */
    public static boolean hasEnoughMemory(Asset asset) {
		return false;
	}

    /**
     * Suggests the minimal max. heap size
     *
     * @param asset asset to check
     * @return minimal max. heap size
     */
    public static long suggestMaxHeapSize(Asset asset) {
		return 0L;
	}

    private static boolean canCalculate(Asset asset) {
		return false;
	}
}
