/*
 * Copyright 1997-2010 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.dam.core.process;

import com.day.cq.dam.api.Asset;
import com.day.cq.dam.api.DamConstants;
import com.day.cq.dam.api.Rendition;
import com.day.cq.dam.commons.process.AbstractAssetWorkflowProcess;
import com.day.cq.dam.core.impl.PrivateConstants;
import com.day.cq.workflow.WorkflowException;
import com.day.cq.workflow.WorkflowSession;
import com.day.cq.workflow.exec.WorkItem;
import com.day.cq.workflow.metadata.MetaDataMap;
import com.day.text.Text;
import org.apache.commons.lang.StringUtils;
import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Property;
import org.apache.felix.scr.annotations.Service;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import static com.day.cq.commons.jcr.JcrConstants.JCR_CONTENT;
import static com.day.cq.commons.jcr.JcrConstants.JCR_LASTMODIFIED;
import static com.day.cq.commons.jcr.JcrConstants.JCR_LAST_MODIFIED_BY;
import static com.day.cq.dam.api.DamConstants.DC_MODIFIED;
import static com.day.cq.dam.api.DamConstants.METADATA_FOLDER;

/**
 * The <code>AssetSetLastModifiedProcess</code> sets the last modified
 * properties of the {@link com.day.cq.dam.api.Asset Asset}. These are the
 * <i>jcr:lastModified</i> and <i>jcr:lastModifiedBy</i>
 * {@link javax.jcr.Property Properties} of Asset's Node and its MetaData <i>
 * {@value com.day.cq.dam.api.DamConstants#DC_MODIFIED}</i> value.
 * <p/>
 * In addition custom properties for the modification date and modifier can be
 * set.
 * <p/>
 * <b>Arguments:</b>
 * <table>
 * <thead>
 * <tr>
 * <td>Prefix</td>
 * <td>Description</td>
 * <td>Example</td>
 * </tr>
 * </thead>
 * <tr>
 * <td>relativeLastModifiedPath:</td>
 * <td>A path pointing to a {@link javax.jcr.Property Property} that may store a
 * {@link java.util.Calendar Calendar}. The Asset's Content Node will be take as
 * root for the relative Path</td>
 * <td>relativeLastModifiedPath:myModificationDate</td>
 * </tr>
 * <tr>
 * <td>relativeLastModifiedByPath:</td>
 * <td>A path pointing to a {@link javax.jcr.Property Property} that may store a
 * String. The Asset's Content Node will be take as root for the relative Path</td>
 * <td>relativeLastModifiedByPath:myModifier</td>
 * </tr>
 * </table>
 *
 * @see AbstractAssetWorkflowProcess
 */
@Component(metatype = false)
@Service
@Property(name = "process.label", value = "Set Last Modified")
public class AssetSetLastModifiedProcess extends AbstractAssetWorkflowProcess {

    /**
     * Logger instance for this class.
     */
    private static final Logger log = LoggerFactory.getLogger(AssetSetLastModifiedProcess.class);

    /**
     * The available arguments to this process implementation.
     */
    public enum Arguments {
        PROCESS_ARGS("PROCESS_ARGS"), RELATIVE_LAST_MODIFIED_PATH("relativeLastModifiedPath"), RELATIVE_LAST_MODIFIED_BY_PATH(
                "relativeLastModifiedByPath");
        private String argumentName;

        Arguments(String argumentName) {
            this.argumentName = argumentName;
        }

        public String getArgumentName() {
            return this.argumentName;
        }

        public String getArgumentPrefix() {
            return this.argumentName + ":";
        }
    }




    public void execute(WorkItem workItem, WorkflowSession workflowSession, MetaDataMap metaData) throws WorkflowException {
	}

    public String[] buildArguments(MetaDataMap metaData) {
		return null;
	}
}
