/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.dam.core.process;

import static com.day.cq.commons.jcr.JcrConstants.JCR_CONTENT;
import static com.day.cq.commons.jcr.JcrConstants.JCR_LASTMODIFIED;
import static com.day.cq.commons.jcr.JcrConstants.JCR_LAST_MODIFIED_BY;
import static com.day.cq.dam.api.DamConstants.PREFIX_ASSET_THUMBNAIL;
import static com.day.cq.dam.api.DamConstants.PREFIX_ASSET_WEB;
import static com.day.cq.dam.core.process.CreateThumbnailProcess.isThumbnailStale;

import java.util.Calendar;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.RepositoryException;

import com.day.cq.dam.commons.util.DamUtil;
import org.apache.commons.lang.StringUtils;
import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Property;
import org.apache.felix.scr.annotations.Reference;
import org.apache.felix.scr.annotations.Service;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.day.cq.dam.api.Asset;
import com.day.cq.dam.api.DamConstants;
import com.day.cq.dam.api.Rendition;
import com.day.cq.dam.api.renditions.RenditionMaker;
import com.day.cq.dam.api.renditions.RenditionTemplate;
import com.day.cq.dam.commons.process.AbstractAssetWorkflowProcess;
import com.day.cq.workflow.WorkflowException;
import com.day.cq.workflow.WorkflowSession;
import com.day.cq.workflow.exec.WorkItem;
import com.day.cq.workflow.metadata.MetaDataMap;

/**
 * The <code>CreateWebEnabledImageProcess</code> is called in a Workflow {@link com.day.cq.workflow.exec.WorkflowProcess Process} step. This
 * Process creates, if possible, a web enabled representation from the {@link com.day.cq.dam.api.Asset Asset}.<br>
 * The Format of the webenabled image, can be set by arguments to the
 * {@link com.day.cq.workflow.exec.WorkflowProcess#execute(WorkItem, WorkflowSession, MetaDataMap) Process}<br>
 * Process is memory aware. If the required memory is not available, image creatation is deferred or cancelled if the memory requirement
 * can't be sattisfied within a fixed amount of trails.
 * <p>
 * <p/>
 * Example with the following workflow step arguments:
 * 
 * <pre>
 *    dimension:500:100,
 *    quality:60
 * </pre>
 * <p/>
 * The Process create a PNG Image of the size of 500x100 Pixels. The Quality will be set to 60%.
 * <p/>
 * <b>Arguments:</b>
 * <table>
 * <thead>
 * <tr>
 * <td>Prefix</td>
 * <td>Description</td>
 * <td>Default</td>
 * <td>Multiple</td>
 * <td>Example</td>
 * </tr>
 * </thead>
 * <tr>
 * <td>dimension:</td>
 * <td>Amount in pixels given in the following format widht:hight.<br>
 * Image will be risized to have at most this dimension, keeping aspect ratio</td>
 * <td>1000:1000</td>
 * <td></td>
 * <td>dimension:400:300</td>
 * </tr>
 * <tr>
 * <td>quality:</td>
 * <td>Quality as percentage from optimal. This depends on the mimetype of the resulting image. For Gifs this reduces the amount of colors,
 * for Jpgs this sets the compression rate, etc.</td>
 * <td>60</td>
 * <td></td>
 * <td>quality:90</td>
 * </tr>
 * <tr>
 * <td>skip:</td>
 * <td>Set the mimetypes of Assets that should not be processed</td>
 * <td></td>
 * <td>mulitple</td>
 * <td>skip:application/pdf, skip:image/tiff</td>
 * </tr>
 * <tr>
 * <td>mimetype:</td>
 * <td>Set the mimetype of the Image to create</td>
 * <td>image/png</td>
 * <td></td>
 * <td>mimetype:image/jpg</td>
 * </tr>
 * <tr>
 * <td>keepFormatList:</td>
 * <td>Commas epareted list of mimetypes that can be taken webinabled image</td>
 * <td>image/pjpeg, image/jpeg, image/jpg, image/gif, image/png, image/x-png</td>
 * <td></td>
 * <td>keepFormatList:image/gif,image/png,image/x-png</td>
 * </tr>
 * </table>
 * 
 * @see AbstractAssetWorkflowProcess
 */
@Component(metatype = false)
@Service
@Property(name = "process.label", value = "Create Web Enabled Image")
public class CreateWebEnabledImageProcess extends AbstractAssetWorkflowProcess {

    private static final Logger log = LoggerFactory.getLogger(CreateWebEnabledImageProcess.class);

    @Reference
    protected RenditionMaker renditionMaker;

    /**
     * Enum mixing old and new metadata argument names.
     */
    public enum Arguments {
        PROCESS_ARGS("PROCESS_ARGS"), // legacy metadata property holding string with all arguments
        DIMENSION("dimension"), // legacy only
        WIDTH("width"), HEIGHT("height"), QUALITY("quality"), MIME_TYPE("mimetype"), KEEP_FORMAT_LIST("keepFormatList"), SKIP("skip");

        public final String legacyName;

        Arguments(String legacyName) {
            this.legacyName = legacyName;
        }
    }

    /** The parsed configuration for this wf process */
    public static class Config {
        public int width;

        public int height;

        public int quality;

        public String mimeType;

        public String[] mimeTypesToKeep;

        public String[] skipMimeTypes;
    }

    public void createWebEnabledImage(WorkItem workItem, Config config, Asset asset, RenditionMaker renditionMaker)
            throws RepositoryException {
			}

    private boolean isWebThumbnailStale(Asset asset, int width, int height) {
		return false;
	}

    public void execute(WorkItem workItem, WorkflowSession workflowSession, MetaDataMap metaData) throws WorkflowException {
	}

    public Config parseConfig(MetaDataMap metaData) {
		return null;
	}

    private Config parseLegacyConfig(String processArgs) {
		return null;
	}

    private String getFirstValueFromArgs(String[] arguments, String key, String defaultValue) {
		return null;
	}

    protected boolean handleAsset(Asset asset, Config config) {
		return false;
	}
}