/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.day.cq.dam.scene7.api.model;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.day.cq.dam.scene7.api.constants.Scene7AssetType;
import com.day.cq.dam.scene7.impl.utils.Scene7AssetUtils;

/**
 * Default implementation for a Scene7Asset
 */
public class Scene7AssetImpl implements Scene7Asset {

    private static final long serialVersionUID = 5536087022446822682L;

    /**
     * Scene7 asset property keys
     */
    public enum Scene7AssetProperty {
        ASSET_TYPE_STRING,
        SCENE7_ASSET_TYPE,
        ASSET_HANDLE,
        NAME,
        FILE_NAME,
        FOLDER,
        FOLDER_HANDLE,
        CREATED_BY,
        MODIFIED_BY,
        VIDEO_ENCODING_PRESET_ID,
        CREATED_DATE,
        MODIFIED_DATE,
        ORIGINAL_FILE,
        ORIGINAL_PATH,
        WIDTH,
        HEIGHT,
        URL_MODIFIER,
        READY_TO_PUBLISH,
        FILE_SIZE,
        VIEWER_PRESET_TYPE,
        VIEWER_PRESET_CONFIGURATION_SETTINGS,
        SUB_ASSETS,
        ORIGINATOR,
        ASSET_PROPERTIES
    }

    /**
     * <code>Map</code> containing the asset's main attributes
     */
    private Map<Object, Object> assetAttributes;

    /**
     * Creates a new <code>Scene7Asset</code> object
     * 
     * @param assetAttributes
     *            a <code>Map</code> object containing the new Scene7 asset's main attributes
     */
    public Scene7AssetImpl(Map<Object, Object> assetAttributes) {

        if (assetAttributes != null) {
            // use the received map only if it is non-null
            this.assetAttributes = assetAttributes;
        } else {
            // create a new map
            this.assetAttributes = new HashMap<Object, Object>();
        }

        // initialize the sub-asset list
        this.assetAttributes.put(Scene7AssetProperty.SUB_ASSETS, new ArrayList<Scene7Asset>());
    }

    /**
     * Copy constructor useful for changing the properties for an already parsed <code>Scene7Asset</code>
     * 
     * @param scene7Asset
     *            the original <code>Scene7Asset</code>
     * @param overrideProperties
     *            a <code>Map</code> containing the properties that need to be updated
     */
    public Scene7AssetImpl(Scene7Asset scene7Asset, Map<Object, Object> overrideProperties) {
        // copy the asset's properties into a map
        Map<Object, Object> originalPropertyMap = getPropertiesMap(scene7Asset);

        // override the properties in the map
        if (overrideProperties != null) {
            originalPropertyMap.putAll(overrideProperties);
        }

        // use the resulting property map for this asset
        this.assetAttributes = originalPropertyMap;
    }

    public Scene7AssetType getAssetType() {
		return null;
	}

    public String getAssetTypeStr() {
		return null;
	}

    public String getName() {
		return null;
	}

    public String getAssetHandle() {
		return null;
	}

    public String getFileName() {
		return null;
	}

    public String getFolder() {
		return null;
	}

    public String getFolderHandle() {
		return null;
	}

    public String getCreatedBy() {
		return null;
	}

    public String getModifiedBy() {
		return null;
	}

    public Date getCreatedDate() {
		return null;
	}

    public Date getModifiedDate() {
		return null;
	}

    public String getOriginalFile() {
		return null;
	}

    public String getOriginalPath() {
		return null;
	}

    /**
     * Returns an unmodifiable view of the internal sub-assets list
     */
    @SuppressWarnings("unchecked")
    public List<Scene7Asset> getSubAssets() {
		return null;
	}

    public Scene7Asset getOriginatorAsset() {
		return null;
	}

    public void updateOriginatorAsset(Scene7Asset originator) {
	}

    @SuppressWarnings("unchecked")
    public void addSubAsset(Scene7Asset subAsset) {
	}

    public Long getWidth() {
		return null;
	}

    public Long getHeight() {
		return null;
	}

    public String getUrlModifier() {
		return null;
	}

    @SuppressWarnings("unchecked")
    public Map<String, String> getViewerPresetConfigurationSettings() {
		return null;
	}

    public String getViewerPresetType() {
		return null;
	}

    public String getRootFolder() {
		return null;
	}

    public boolean isPublished() {
		return false;
	}

    public Integer getFileSize() {
		return null;
	}

    public String getVideoEncodingPresetId() {
		return null;
	}

    /**
     * Returns one of the properties stored in a property map, while also converting to a specified type if possible
     * 
     * @param map
     *            the map from which to extract the property value
     * @param key
     *            the key under which the data is stored
     * @param convertTo
     *            the type to which conversion will be attempted
     * 
     * @return a property converted to the requested type if successful, null otherwise
     */
    private <T> T getProperty(Map<?, ?> map, Object key, Class<T> convertTo) {
		return null;
	}

    /**
     * Creates a <code>Map</code> that contains the properties of the received <code>Scene7Asset</code>
     * 
     * @param scene7Asset
     *            a <code>Scene7Asset</code> implementation
     * 
     * @return a <code>Map</code> containing the properties of the Scene7 asset
     */
    private Map<Object, Object> getPropertiesMap(Scene7Asset scene7Asset) {
		return null;
	}

    @SuppressWarnings("unchecked")
    public Map<String, String> getAssetProperties() {
		return null;
	}

}
