/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.day.cq.dam.word.extraction;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Properties;
import org.apache.felix.scr.annotations.Property;
import org.apache.felix.scr.annotations.Service;
import org.apache.poi.hwpf.HWPFDocument;
import org.apache.poi.hwpf.model.PicturesTable;
import org.apache.poi.hwpf.usermodel.CharacterRun;
import org.apache.poi.hwpf.usermodel.Paragraph;
import org.apache.poi.hwpf.usermodel.Picture;
import org.apache.poi.hwpf.usermodel.Range;
import org.apache.poi.util.IOUtils;
import org.apache.poi.xwpf.usermodel.BodyElementType;
import org.apache.poi.xwpf.usermodel.VerticalAlign;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.apache.poi.xwpf.usermodel.XWPFParagraph;
import org.apache.poi.xwpf.usermodel.XWPFPicture;
import org.apache.poi.xwpf.usermodel.XWPFPictureData;
import org.apache.poi.xwpf.usermodel.XWPFRun;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.wrappers.ValueMapDecorator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.day.cq.dam.api.Asset;
import com.day.cq.dam.api.Rendition;
import com.day.cq.dam.api.RenditionPicker;
import com.day.cq.dam.indd.AbstractPageExtractionHandler;
import com.day.cq.dam.indd.PageBuilder;
import com.day.cq.dam.indd.PageComponent;
import com.day.cq.dam.indd.PageExtractionException;
import com.day.cq.dam.indd.PageExtractionHandler;
import com.day.cq.wcm.api.Page;
import com.day.cq.wcm.api.WCMException;

/**
 * A {@link PageExtractionHandler} for Word's DOCX and DOC format.
 * <p/>
 * An extraction handler to extract a page from a Word .docx and .doc files.
 *
 * @see PageExtractionHandler
 */

@Component
@Service(value = PageExtractionHandler.class)
@Properties({
	@Property(name = PageExtractionHandler.SERVICE_PROPERTY_LABEL, value = "Word Extraction Handler", propertyPrivate = true),
	@Property(name = PageExtractionHandler.SERVICE_PROPERTY_DESCRIPTION, value = "Extraction Handler for a MS Word files.", propertyPrivate = true)
})
public class WordExtractionHandler extends AbstractPageExtractionHandler {
	
	private static final Logger log = LoggerFactory.getLogger(WordExtractionHandler.class);
	
	private static final String TEXT_COMPONENT = "foundation/components/text";
	private static final String IMAGE_COMPONENT = "foundation/components/image";
	private static final String TEXT_IMAGE_COMPONENT = "foundation/components/textimage";
	
	private static final int JUSTIFICATION_RIGHT = 2;
	private static final int JUSTIFICATION_CENTER = 1;
	
	private static final short TEXT_TYPE = 0;
	private static final short IMAGE_TYPE = 1;
	private static final short TEXT_IMAGE_TYPE = 2;
	
	/**
     * Get an MS Word targeted rendition picker.
     * <p/>
     * {@inheritDoc}
     */
	public RenditionPicker getRenditionPicker() {
		return null;
	}
	
	/**
     * {@inheritDoc}
     */
	public Page extractPage(Rendition rend, String pageRoot, String pageName, String pageTitle, String pageTemplate, String pageDesign) 
			throws PageExtractionException {
				return null;
			}
	
	/**
	 * Extract the text, image, and textimage components from a Word .doc file and create a CQ Page that contains them plus a title.
	 * @param rend
	 * @param pageRoot
	 * @param pageName
	 * @param pageTitle
	 * @param pageTemplate
	 * @param pageDesign
	 * @return
	 * @throws PageExtractionException
	 */
	private Page extractDoc(Rendition rend, String pageRoot, String pageName, String pageTitle, String pageTemplate, String pageDesign) 
			throws PageExtractionException {
				return null;
			}
	
	/**
	 * Extract the text, image, and textimage components from a Word .docx file and create a CQ Page that contains them plus a title.
	 * @param rend
	 * @param pageRoot
	 * @param pageName
	 * @param pageTitle
	 * @param pageTemplate
	 * @param pageDesign
	 * @return
	 * @throws PageExtractionException
	 */
	private Page extractDocx(Rendition rend, String pageRoot, String pageName, String pageTitle, String pageTemplate, String pageDesign)
			throws PageExtractionException {
				return null;
			}

	private Page buildPage(Rendition rend, String pageRoot, String pageName,
			String pageTitle, String pageTemplate, String pageDesign,
			PageBuilder pageBuilder, PageComponent par) throws WCMException {
				return null;
			}
	
	@Override
	protected PageComponent createTitleComponent(PageBuilder pageBuilder, String pageTitle) {
		return null;
	}
	
	/**
	 * Given the paragraph from a Word .doc file, 1 textimage component is created.  Any additional images in the paragraph, beyond the first
	 * are created as image components.
	 * @param para
	 * @param pageBuilder
	 * @param name
	 * @param subAssetPath
	 * @param picTable
	 * @return A list of the components to be added to the Page.
	 */
	private List<PageComponent> createTextImageNode(Paragraph para, PageBuilder pageBuilder, String name, String subAssetPath,
			PicturesTable picTable) {
				return null;
			}
	/**
	 * Given the paragraph from a Word .docx file, 1 textimage component is created.  Any additional images in the paragraph, beyond the first
	 * are created as image components.
	 * @param para
	 * @param pageBuilder
	 * @param name
	 * @param subAssetPath
	 * @return A list of the components to be added to the Page.
	 */
	private List<PageComponent> createTextImageNode(XWPFParagraph para, PageBuilder pageBuilder, String name, String subAssetPath) {
		return null;
	}
	
	/**
	 * Given the paragraph from a Word .doc, image components are created for each image found in the paragraph.
	 * @param para
	 * @param pageBuilder
	 * @param name
	 * @param subAssetPath
	 * @param picTable
	 * @return A list of components to be added to the page.
	 */
	private List<PageComponent> createImageNode(Paragraph para, PageBuilder pageBuilder, String name, String subAssetPath, 
			PicturesTable picTable) {
				return null;
			}
	
	/**
	 * Given the paragraph from a Word .docx, image components are created for each image found in the paragraph.
	 * @param para
	 * @param pageBuilder
	 * @param name
	 * @param subAssetPath
	 * @return A list of components to be added to the page.
	 */
	private List<PageComponent> createImageNode(XWPFParagraph para, PageBuilder pageBuilder, String name, String subAssetPath) {
		return null;
	}
	
	/**
	 * Given the paragraph from a Word .doc, create a text component to emcompass the paragraph text.
	 * @param para
	 * @param pageBuilder
	 * @param name
	 * @return The text component to be added to the page.
	 */
	private PageComponent createTextNode(Paragraph para, PageBuilder pageBuilder, String name) {
		return null;
	}
	
	/**
	 * Given the paragraph from a Word .docx, create a text component to emcompass the paragraph text.
	 * @param para
	 * @param pageBuilder
	 * @param name
	 * @return The text component to be added to the page.
	 */
	private PageComponent createTextNode(XWPFParagraph para, PageBuilder pageBuilder, String name) {
		return null;
	}
	
	private void addOpenStyleTags(XWPFRun run, StringBuilder builder) {
	}
	
	private void addCloseStyleTags(XWPFRun run, StringBuilder builder) {
	}
	
	private int checkStyle(XWPFParagraph para) {
		return 0;
	}
	
	private Paragraph[] getParagraphs(Range range) {
		return null;
	}
	
	private short getParagraphType(XWPFParagraph para) {
		return 0;
	}
	
	private short getParagraphType(Paragraph paragraph, HWPFDocument doc) {
		return 0;
	}
	
	/**
	 * Takes the given paragraph from a Word .doc file and formats the text with proper HTML tags.
	 * @param paragraph
	 * @return The HTML formatted text.
	 */
	private String renderParagraph(Paragraph paragraph) {
		return null;
	}
	
	private String renderParagraph(XWPFParagraph para) {
		return null;
	}
	
}
