/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.wcm.commons;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Calendar;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletResponse;

import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.SlingHttpServletResponse;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.ValueMap;
import org.apache.sling.api.servlets.SlingSafeMethodsServlet;

import com.day.cq.commons.DiffInfo;
import com.day.cq.commons.DiffService;
import com.day.cq.commons.ImageResource;
import com.day.cq.commons.SlingRepositoryException;
import com.day.cq.commons.jcr.JcrConstants;
import com.day.cq.wcm.api.Page;
import com.day.cq.wcm.api.PageManager;
import com.day.cq.wcm.api.components.Component;
import com.day.cq.wcm.api.designer.Style;
import com.day.image.Layer;

/**
 * Servers as base for image servlets
 */
public abstract class AbstractImageServlet extends SlingSafeMethodsServlet {

    /**
     * Handles a GET request and created the desired image.
     * calls the following methods:
     *
     * <ul>
     * <li>{@link #checkModifiedSince(SlingHttpServletRequest, SlingHttpServletResponse)}
     * <li>{@link #createLayer(ImageContext)}
     * <li>{@link #writeLayer(SlingHttpServletRequest, SlingHttpServletResponse, ImageContext, Layer)}
     * </ul>
     *
     * If the requested extension does not map to a known image type via
     * {@link #getImageType(String)}, a 404 is responded.
     */
    protected void doGet(SlingHttpServletRequest request,
                         SlingHttpServletResponse response)
            throws ServletException, IOException {
			}

    /**
     * Writes the layer to the response.
     *
     * @param request servlet request
     * @param response servlet response
     * @param context the context
     * @param layer layer
     * @throws IOException if an I/O error occurs.
     * @throws RepositoryException if an repository error occurs.
     */
    protected void writeLayer(SlingHttpServletRequest request,
                         SlingHttpServletResponse response,
                         ImageContext context, Layer layer)
            throws IOException, RepositoryException {
			}

    /**
     * Writes the layer to the response at the specified quality.
     *
     * @param request servlet request
     * @param response servlet response
     * @param context the context
     * @param layer layer
     * @param quality image quality
     * @throws IOException if an I/O error occurs.
     * @throws RepositoryException if an repository error occurs.
     */
    protected void writeLayer(SlingHttpServletRequest request,
    SlingHttpServletResponse response,
    ImageContext context, Layer layer, double quality)
            throws IOException, RepositoryException {
			}

    /**
     * Returns the image type. default "image/png"
     * @return the image type.
     */
    protected String getImageType() {
		return null;
	}

    /**
     * Returns the image type for the given extension. currently there are only
     * "png", "gif" and "jpg" supported, but an subclass can provide other
     * mappings.
     *
     * @param ext the extension
     * @return the image type or <code>null</code>.
     */
    protected String getImageType(String ext) {
		return null;
	}

    /**
     * Returns the image quality. default 1.0
     * @return the image quality.
     */
    protected double getImageQuality() {
		return 0.0d;
	}

    /**
     * Checks if the request contains a if-last-modified-since header and if the
     * node has a jcr:lastModified property. if the properties were modified
     * before the header a 304 is sent otherwise the response last modified header
     * is set. If the give doesn't have the property, the parent node is searched.
     *
     * @param req the request
     * @param resp the response
     * @return <code>true</code> if the response was sent
     */
    protected boolean checkModifiedSince(SlingHttpServletRequest req,
                                         SlingHttpServletResponse resp) {
											return false;
										}

    /**
     * Creates the image layer.
     *
     * @param c the convenience context
     * @return the layer
     * @throws RepositoryException if an error occurs.
     * @throws IOException if an I/O error occurs
     */
    protected abstract Layer createLayer(ImageContext c)
            throws RepositoryException, IOException {
		return null;
	}

    /**
     * Default behavior that applies diff information to the layer
     * @param layer the layer
     * @param c the context
     * @return <code>true</code> if the layer was modified.
     */
    protected boolean applyDiff(Layer layer, ImageContext c) {
		return false;
	}

    /**
     * Creates an image resource based on the resource in the image context.
     * Subclasses can override this to create extended variants of image
     * resources.
     *
     * @param resource the resource
     * @return the image resource
     */
    protected ImageResource createImageResource(Resource resource) {
		return null;
	}

    /**
     * Calculates if the underlying image was modified in respect to the versioned
     * diff.
     *
     * @param c the image context
     * @return <code>true</code> if modified
     */
    protected boolean isModifiedDiff(ImageContext c) {
		return false;
	}

    /**
     * Calculates if the underlying image was added with respect to the versioned
     * diff.
     *
     * @param c the image context
     * @return <code>true</code> if added
     */
    protected boolean isAddedDiff(ImageContext c) {
		return false;
	}

    /**
     * Calculates if the underlying image was removed with respect to the versioned
     * diff.
     *
     * @param c the image context
     * @return <code>true</code> if removed
     */
    protected boolean isRemovedDiff(ImageContext c) {
		return false;
	}

    /**
     * Convenience class that holds useful stuff needed for image generation
     */
    public static class ImageContext {

        public final SlingHttpServletRequest request;

        public final Resource resource;

        public final Resource defaultResource;

        public final ResourceResolver resolver;

        public final Node node;

        public final ValueMap properties;

        public final Style style;

        public final Page currentPage;

        public final ValueMap pageProperties;

        public final Component component;

        public final String requestImageType;

        public final DiffInfo diffInfo;

        public ImageContext(SlingHttpServletRequest request, String type) {
            this.request = request;
            resource = request.getResource();
            resolver = request.getResourceResolver();
            node = resource.adaptTo(Node.class);
            ValueMap props = resource.adaptTo(ValueMap.class);
            properties = props == null ? ValueMap.EMPTY : props;
            style = WCMUtils.getStyle(request);

            PageManager pageManager = resolver.adaptTo(PageManager.class);
            currentPage = pageManager.getContainingPage(resource);
            pageProperties = currentPage == null ? ValueMap.EMPTY : currentPage.getProperties();
            component = WCMUtils.getComponent(resource);
            requestImageType = type;

            // get diff information
            Resource vRes = null;
            DiffInfo.TYPE diffType = null;
            String vType = request.getParameter(DiffService.REQUEST_PARAM_DIFF_TYPE);
            if (vType != null) {
                try {
                    diffType = DiffInfo.TYPE.valueOf(vType);
                } catch (IllegalArgumentException e) {
                    // ignore
                }
            }
            String vLabel = request.getParameter(DiffService.REQUEST_PARAM_DIFF_TO);
            if (vLabel != null) {
                vRes = DiffInfo.getVersionedResource(resource, vLabel);
                if (vRes == null && diffType == null) {
                    // calculate missing vType
                    if (!resource.getPath().contains(JcrConstants.JCR_FROZENNODE)) {
                        diffType = DiffInfo.TYPE.ADDED;
                    }
                }
            }

            String defaultResourcePath = request.getParameter(ImageResource.PN_DEFAULT_IMAGE_PATH);
            Resource defRes = null;
            if (defaultResourcePath != null) {
                defRes = resolver.resolve(defaultResourcePath);
            }
            defaultResource = defRes;

            //
            if (vRes != null || diffType != null) {
                diffInfo = new DiffInfo(vRes, diffType == null ? DiffInfo.TYPE.SAME : diffType);
            } else {
                diffInfo = null;
            }
        }
    }
}