/*
 * Copyright 1997-2009 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.wcm.foundation.forms;

import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceUtil;
import org.apache.sling.api.resource.ValueMap;

import com.day.cq.wcm.foundation.forms.impl.FormsUtil;


/**
 * A description of a form field as it is later used in the html
 * form.
 * A field component usually maps to a single field description,
 * however compound fields like an address field map to
 * several field descriptions.
 *
 * The field description is used mostly during validation as it
 * contains all necessary information.
 *
 * Field descriptions can be maintained from the {@link FieldHelper}.
 * @since 5.3
 */
public class FieldDescription {

    /** Field name. */
    private String name;

    /** Is this field required? */
    private boolean required = false;

    /** The message which is displayed if a required field is not set. */
    private String requiredMsg;

    /** Optional constraint type */
    private String constraintType;

    /** The message which is displayed if the constraint is not met. */
    private String constraintMsg;

    /** Is this a read only field? */
    private boolean readOnly = false;

    /** Is this a private field? */
    private boolean privateField = false;

    /** Is this a multi value field? */
    private boolean multiValue = false;

    /** The associated resource. */
    private final Resource fieldResource;

    /**
     * Construct a new field description for a field resource.
     * @param rsrc
     */
    public FieldDescription(final Resource rsrc) {
        this.fieldResource = rsrc;
    }

    /**
     * Construct a new field description for a field resource
     * and set the name property.
     * @param rsrc The resource
     * @param name The field name.
     */
    public FieldDescription(final Resource rsrc, final String name) {
        this.fieldResource = rsrc;
        this.name = name;
    }

    public void update(final Resource rsrc) {
	}

    /**
     * Return the associated field resource
     */
    public Resource getFieldResource() {
		return null;
	}

    /**
     * Return the name of the field.
     */
    public String getName() {
		return null;
	}

    /**
     * Set the name for the field.
     */
    public void setName(String name) {
	}

    /**
     * Is this field required?
     */
    public boolean isRequired() {
		return false;
	}

    /**
     * Set if this field is requried.
     */
    public void setRequired(boolean required) {
	}

    /**
     * Return the error message for a required field.
     */
    public String getRequiredMessage() {
		return null;
	}

    /**
     * Set the error message for a required field.
     */
    public void setRequiredMessage(String requiredMsg) {
	}

    /**
     * Get the constraint type (might be null).
     */
    public String getConstraintType() {
		return null;
	}

    /**
     * Set the constraint type (might be null).
     */
    public void setConstraintType(String constraintType) {
	}

    /**
     * The error message if the constraint is not met.
     */
    public String getConstraintMessage() {
		return null;
	}

    /**
     * Set error message if the constraint is not met.
     */
    public void setConstraintMessage(String constraintMsg) {
	}

    /**
     * Is this field readonly?
     */
    public boolean isReadOnly() {
		return false;
	}

    /**
     * Set if this field is readonly.
     */
    public void setReadOnly(boolean readOnly) {
	}

    /**
     * Is this a private field?
     */
    public boolean isPrivate() {
		return false;
	}

    /**
     * Set if this is a private field.
     */
    public void setPrivateField(boolean flag) {
	}

    /**
     * Is this a multi value field?
     */
    public boolean isMultiValue() {
		return false;
	}

    /**
     * Set if this is a private field.
     */
    public void setMultiValue(boolean flag) {
	}
}
