/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2017 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.day.cq.wcm.foundation.model.responsivegrid;

import com.adobe.cq.export.json.ComponentExporter;
import com.day.cq.wcm.api.NameConstants;
import com.day.cq.wcm.api.designer.ComponentStyle;
import com.day.cq.wcm.commons.WCMUtils;
import com.day.cq.wcm.foundation.model.responsivegrid.export.ResponsiveColumnExporter;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.resource.AbstractResource;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceMetadata;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.ValueMap;
import org.apache.sling.models.factory.ModelFactory;
import org.osgi.annotation.versioning.ProviderType;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Represents a column of a responsive grid.
 * A responsive grid's column is basically a virtual wrapper of a component that is part of the grid.
 */
@ProviderType
@JsonSerialize(as = ResponsiveColumnExporter.class)
public class ResponsiveColumn extends AbstractResource implements ResponsiveColumnExporter {

    private Resource wrappedResource;

    private ComponentExporter componentModel;

    private String classNames;

    private ResponsiveConfiguration responsiveConfiguration = new ResponsiveConfiguration();

    private Set<String> columnOnlyBreakpointNames;
    private Map<String, Breakpoint> breakpoints;
    private Map<String, Breakpoint> gridBreakpoints;

    public ResponsiveColumn(Resource resource,
                            Map<String, Breakpoint> gridBreakpoints,
                            String classNamePrefix) {
        this.wrappedResource = resource;

        Map<String, Breakpoint> breakpoints = getBreakpoints(this.wrappedResource, gridBreakpoints);
        this.classNames = createClassNames(
                this.wrappedResource,
                classNamePrefix + ResponsiveConstants.DEFAULT_COLUMN_CSS_PREFIX,
                breakpoints,
                gridBreakpoints);
        this.gridBreakpoints = gridBreakpoints;
        this.responsiveConfiguration = new ResponsiveConfiguration(breakpoints);
    }

    public ResponsiveColumn(Resource resource,
                                Map<String, Breakpoint> gridBreakpoints,
                                String classNamePrefix,
                                SlingHttpServletRequest slingRequest,
                                ModelFactory modelFactory) {
        this.wrappedResource = resource;
        this.componentModel = modelFactory.getModelFromWrappedRequest(slingRequest, resource, ComponentExporter.class);

        Map<String, Breakpoint> breakpoints = getBreakpoints(this.wrappedResource, gridBreakpoints);
        this.classNames = createClassNames(
                this.wrappedResource,
                classNamePrefix + ResponsiveConstants.DEFAULT_COLUMN_CSS_PREFIX,
                breakpoints,
                gridBreakpoints);
        this.gridBreakpoints = gridBreakpoints;
        this.responsiveConfiguration = new ResponsiveConfiguration(breakpoints);
    }

    @Override
    public Map<String, Breakpoint> getBreakpoints() {
		return null;
	}

    @Override
    public String getColumnClassNames() {
		return null;
	}

    /**
     *
     * @deprecated Use {@link #getColumnClassNames()}
     */
    public String getCssClass() {
		return null;
	}

    /**
     *
     * @deprecated Use {@link #getBreakpoints()}
     */
    public Set<String> getMissingBreakpointNames() {
		return null;
	}

    /**
     * Returns the number of columns for the given breakpoint name
     * @param breakpointName The name of the breakpoint
     * @return The number of columns for given breakpoint name.
     * @deprecated
     */
    public Integer getColumnCount(String breakpointName) {
		return null;
	}

    @Nonnull
    @Override
    public ComponentExporter getExportedComponent() {
		return null;
	}

    @Override
    @Nonnull
    public String getExportedType() {
		return null;
	}

    /**
     *
     * @return The wrapped {@link Resource} associated with the column.
     */
    @Nonnull
    public Resource getResource() {
		return null;
	}

    /**
     *
     * @return The path of the wrapped {@link Resource} associated with the column.
     */
    @Nonnull
    public String getPath() {
		return null;
	}

    /**
     *
     * @deprecated
     */
    @Nonnull
    public String getResourceType() {
		return null;
	}

    /**
     *
     * @deprecated
     */
    public String getResourceSuperType() {
		return null;
	}

    /**
     *
     * @deprecated
     */
    @Nonnull
    public ResourceMetadata getResourceMetadata() {
		return null;
	}

    /**
     *
     * @deprecated
     */
    @Nonnull
    public ResourceResolver getResourceResolver() {
		return null;
	}

    /**
     *
     * @deprecated
     */
    public ValueMap getProperties() {
		return null;
	}

    /**
     * Generates the class names to be applied to the column
     */
    private String createClassNames(Resource resource,
                                    String columnCssClassPrefix,
                                    Map<String, Breakpoint> columnBreakpoints,
                                    Map<String, Breakpoint> referenceBreakpoints) {
										return null;
									}

    /**
     *
     * @param resource The current resource
     * @param referenceBreakpoints The reference breakpoints (breakpoints of the grid parent)
     * @return Returns a map of processed breakpoints. If necessary, the breakpoints of the current resource are adjusted with the ones of the given reference (often the grid parent).
     */
    private Map<String, Breakpoint> getBreakpoints(Resource resource, Map<String, Breakpoint> referenceBreakpoints) {
		return null;
	}

}