/*
 * $Id: IO.java 12345 2004-08-22 04:56:09Z fielding $
 *
 * Copyright 1997-2004 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.io;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.CharArrayWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Reader;
import java.io.UTFDataFormatException;
import java.io.Writer;
import java.util.Collection;

/**
 * Contains I/O utility functions.
 *
 * @version $Revision: 1.15 $, $Date: 2004-08-22 06:56:09 +0200 (Sun, 22 Aug 2004) $
 * @author tripod
 * @since antbear
 * @audience wad
 */
public class IO {

    /** default log category */
    private static final Logger log = LoggerFactory.getLogger(IO.class);

    /** temporary directory name */
    private static final String TEMPDIR = "tmp";

    /** Temporary directory folder */
    private static File tempDir;

    /** the current work directory file */
    private static File cwd;

    /**
     * Spools data from an <code>InputStream</code> into an
     * <code>OutputStream</code>. Please note, that both, input and output
     * stream, remain open after the spooling.
     *
     * @param in
     *            input stream
     * @param out
     *            output stream
     *
     * @return number of bytes transmitted
     *
     * @throws IOException
     *             if an I/O error occurs
     */
    public static int spool(InputStream in, OutputStream out)
            throws IOException {
				return 0;
			}

    /**
     * Spools data from an <code>InputStream</code> into an
     * byte buffer. either until the buffer is full or the EOF is reached.
     *
     * @param in
     *            input stream
     * @param buffer
     *            output buffer
     *
     * @return number of bytes transmitted
     *
     * @throws IOException
     *             if an I/O error occurs
     */
    public static int spool(InputStream in, byte[] buffer)
            throws IOException {
				return 0;
			}

    /**
     * Spools characters from an <code>Reader</code> into an
     * <code>Writer</code>. Please note, that both, reader and writer remain
     * open after the spooling.
     *
     * @param in
     *            input Reader
     * @param out
     *            output Writer
     *
     * @return number of characters transmitted
     *
     * @throws IOException
     *             if an I/O error occurs
     */
    public static int spool(Reader in, Writer out) throws IOException {
		return 0;
	}

    /**
     * Spools data from an <code>InputStream</code> into an
     * <code>OutputStream</code>. Please note, that both, input and output
     * stream, remain open after the spooling. This method only transfers
     * <code>num</code> bytes or less.
     *
     * @param in
     *            input stream
     * @param out
     *            output stream
     *
     * @return number of bytes transmitted
     *
     * @throws IOException
     *             if an I/O error occurs
     */
    public static int spool(InputStream in, OutputStream out, int num)
        throws IOException {
			return 0;
		}

    /**
     * Closes the <code>InputStream</code> <code>in</code> if not
     * <code>null</code> and ignores a potential <code>IOException</code> thrown
     * from closing the stream.
     *
     * @param in The <code>InputStream</code> to close. This may be
     *          <code>null</code>.
     */
    public static void tryClose(InputStream in) {
	}

    /**
     * Closes the <code>OutputStream</code> <code>out</code> if not
     * <code>null</code> and ignores a potential <code>IOException</code> thrown
     * from closing the stream.
     *
     * @param out The <code>OutputStream</code> to close. This may be
     *          <code>null</code>.
     */
    public static void tryClose(OutputStream out) {
	}

    /**
     * Closes the <code>Reader</code> <code>out</code> if not
     * <code>null</code> and ignores a potential <code>IOException</code> thrown
     * from closing the reader.
     *
     * @param reader The <code>Reader</code> to close. This may be
     *          <code>null</code>.
     */
    public static void tryClose(Reader reader) {
	}

    /**
     * Closes the <code>Writer</code> <code>in</code> if not
     * <code>null</code> and ignores a potential <code>IOException</code> thrown
     * from closing the writer.
     *
     * @param writer The <code>Writer</code> to close. This may be
     *          <code>null</code>.
     */
    public static void tryClose(Writer writer) {
	}

    /**
     * Renames a file to another. If the renaming does not work, it copies the
     * content and tries to delete the source file.
     *
     * @param src
     *            the source file
     * @param dst
     *            the destination file
     *
     * @return <code>true</code> if the renaming was successful and the file
     *         at leas exists at the new location; <code>false</code> if the
     *         renaming failed. in this case, the original file still exists.
     */
    public static boolean rename0(File src, File dst) {
		return false;
	}

    /**
     * Read a UTF encoded string from an input stream.
     *
     * @param in
     *            input stream
     * @return string
     * @throws UTFDataFormatException
     *             if the input stream contains an invalid combination of bytes
     * @throws IOException
     *             if an I/O error occurs
     */
    public static String readUTF(InputStream in) throws UTFDataFormatException,
        IOException {
			return null;
		}

    /**
     * Initialization of CQ3 home. Either this method or
     * {@link #setTempDir(File)} must be called before methods inside {@link IO}
     * may be used.
     *
     * @param cwd directory to use as current work directory
     * @audience core
     */
    public static void setCWD(File cwd) {
	}

    /**
     * Initialization of CQ3 home. Either this method or
     * {@link #setTempDir(File)} must be called before methods inside {@link IO}
     * may be used.
     *
     * @param cq3Home directory to use as CQ3 home.
     * @deprecated use {@link IO#setCWD(File)} instead.
     */
    public static void setCQ3Home(File cq3Home) {
	}

    /**
     * Initialization of temporary directory when used as a separate component.
     * This class has some functionality based on a temporary directory to use
     * and in order to use that functionality without having an engine at hand
     * makes this method necessary.
     *
     * @param tempDir
     *            temporary directory to use
     * @audience core
     */
    public static void setTempDir(File tempDir) {
	}

    /**
     * Return the temporary directory.
     *
     * @return the temporary directory.
     */
    public static File getTempDir() {
		return null;
	}

    /**
     * Returns the path of the application home directory
     *
     * @return the path of the application home directory
     * @deprecated use {@link IO#getCWD()}.{@link File#getPath() getPath()} instead.
     */
    public static String getAppHome() {
		return null;
	}

    /**
     * Returns the directory of the application home
     *
     * @return the directory of the application home
     * @deprecated  use {@link IO#getCWD()} instead.
     */
    public static File getAppHomeDirectory() {
		return null;
	}

    /**
     * Returns the directory of the application home
     *
     * @return the directory of the application home
     */
    public static File getCWD() {
		return null;
	}

    /**
     * Normalize a path, converting slashes to platform dependent separators and
     * removing dots.
     *
     * @return canonical file path or <code>null</code> if the path cannot be
     *         normalized
     */
    public static File getCanonicalFile(String path) {
		return null;
	}

    /**
     * Return a flag indicating whether a file is the parent of another file
     *
     * @param parent
     *            parent file
     * @param child
     *            child file
     * @return <code>true</code> if parent is parent of child, otherwise
     *         <code>false</code>
     */
    public static boolean isParent(File parent, File child) {
		return false;
	}

    /**
     * Create a temporary file and add it to the given <code>tracker</code>
     *
     * @param tracker collection that is use to record the generated temp file.
     * @return a new temporary file.
     * @throws IOException if an I/O error occurs
     */
    public static File createTempFile(Collection tracker)
        throws IOException {
			return null;
		}

    /**
     * Create a temporary file and add it to the given <code>tracker</code>
     *
     * @param tracker collection that is use to record the generated temp file.
     * @param prefix the prefix for the temporary file
     * @param suffix the suffix for the temporary file
     *
     * @return a new temp file
     * @throws IOException if an I/O error occurs
     */
    public static File createTempFile(Collection tracker, String prefix,
        String suffix) throws IOException {
			return null;
		}

    /**
     * Create a temporary file in the generic tmp directory.
     *
     * @return a new temporary file
     * @throws IOException if an I/O error occurs
     */
    public static File createTempFile() throws IOException {
		return null;
	}

    /**
     * Create a temporary file in the generic tmp directory.
     *
     * @param prefix
     *            the prefix for the file
     * @param suffix
     *            the suffix for the file
     *
     * @return a new temporary file
     *
     * @throws IllegalArgumentException
     *             If the prefix argument contains fewer than three characters
     * @throws IOException
     *             If a file could not be created
     */
    public static File createTempFile(String prefix, String suffix)
        throws IOException {
			return null;
		}

    /**
     * returns an absolute path regarding the cq3 home as cwd. If the
     * <code>path</code> already is absolute, this string itself is returned.
     *
     * @param path
     *            the relative path
     *
     * @return an absolute path
     */
    public static String getAbsolutePath(String path) {
		return null;
	}

    /**
     * returns an absolute file regarding the cq3 home as cwd.
     *
     * @param path
     *            the relative path
     *
     * @return a new file pointing to the absolute path
     */
    public static File getAbsoluteFile(String path) {
		return null;
	}

    /**
     * Renames a file to another. If the renaming does not work, it copies the
     * content and tries to delete the source file.
     *
     * @param src
     *            the source file
     * @param dst
     *            the destination file
     *
     * @return <code>true</code> if the renaming was successful and the file
     *         at leas exists at the new location; <code>false</code> if the
     *         renaming failed. in this case, the original file still exists.
     */
    public static boolean rename(File src, File dst) {
		return false;
	}
}
