/*
 * Copyright (c) 1997-2009 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.io.disk;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.StringTokenizer;

/**
 * An utility class to measure the free disk space and the current ulimit.
 * To measure the disk space, it uses the Java 6 method File.getUsableSpace if available,
 * or the command "df" (Unix) or "dir" (Windows).
 * To check the ulimit (Unix only), it runs the command "ulimit -n".
 */
public class DiskSpaceUtil {

    /**
     * If this system property is set, this command line is used to measure the free disk space.
     */
    public static final String COMMAND_DISK_SPACE = "crx.diskSpaceMeasureCommand";

    /**
     * If this system property is set, this command line is used to measure the ulimit.
     */
    public static final String COMMAND_ULIMIT = "crx.ulimitMeasureCommand";

    /**
     * The maximum measure interval  in milliseconds. The default value 60000
     * means measure once a minute.
     * If -1, disk space is never measured.
     */
    public static final String INTERVAL = "crx.diskSpaceMeasureInterval";

    private static final DiskSpaceUtil INSTANCE = new DiskSpaceUtil();

    private long defaultMeasureInterval = Long.decode(System.getProperty(INTERVAL, "60000")).longValue();

    private boolean unsupportedFileUsableSpace;
    private boolean unsupportedDiskFree;

    private long lastMeasuredTime;
    private long lastMeasuredAvailableSpace;

    private int maxOpenFiles;

    /**
     * Get the instance.
     *
     * @return the instance
     */
    public static DiskSpaceUtil getInstance() {
		return null;
	}

    /**
     * Get the maximum number of open files for this process.
     * Under Linux it uses the command "ulimit". When running in Windows,
     * the value Integer.MAX_VALUE is returned.
     *
     * @return the maximum number of open files, or Integer.MAX_VALUE
     */
    public int getMaxOpenFiles() {
		return 0;
	}

    /**
     * Get the available disk space in megabytes for the current working
     * directory using the default interval.
     *
     * @return the space or -1 if it can't be measured
     */
    public long getAvailableDiskSpaceMB() {
		return 0L;
	}

    /**
     * Get the available disk space in megabytes for the given directory using
     * the default interval.
     *
     * @param dir the directory
     * @return the space or -1 if it can't be measured
     */
    public long getAvailableDiskSpaceMB(File dir) {
		return 0L;
	}

    /**
     * Get the available disk space in megabytes for the given directory and
     * using the given interval.
     *
     * @param dir the directory
     * @param interval the interval in milliseconds
     * @return the space in megabytes or -1 if it can't be measured;
     *  Integer.MAX_VALUE if the interval is smaller than 0
     */
    public long getAvailableDiskSpaceMB(File dir, long interval) {
		return 0L;
	}

    /**
     * Call the Java method File.getUsableSpace if available. If not, call
     * diskFree.
     *
     * @param dir the directory
     * @return -1 or the number of megabytes available
     */
    private long getUsableSpace(File dir) {
		return 0L;
	}

    /**
     * Check whether this is a Windows system.
     *
     * @return true if it is
     */
    private boolean isWindows() {
		return false;
	}

    /**
     * Call the Unix command "df".
     *
     * @param dir the directory
     * @return -1 or the number of megabytes available
     */
    private long diskFree(File dir) {
		return 0L;
	}

    /**
     * Try to parse the result of "df -m ." / "dir" and return the free disk
     * space in MB.
     *
     * @param result the result of "df -m ." / "dir".
     * @param isWindows whether "dir" was executed
     * @return the free disk space or -1 if parsing failed
     */
    private long parseResult(String result, boolean isWindows) {
		return 0L;
	}

    /**
     * Tokenize a string.
     *
     * @param s the string
     * @param delimiters the delimiters
     * @return the array list
     */
    private static List tokenize(String s, String delimiters) {
		return null;
	}

    /**
     * Run a process and return the result.
     *
     * @param args the command line
     * @return the result
     */
    private String runProcess(List args) {
		return null;
	}

    private void logDebug(String message) {
	}

    private Thread copyInThread(final InputStream in, final OutputStream out, final boolean toSysOut) {
		return null;
	}

    public long getDefaultMeasureInterval() {
		return 0L;
	}

}
