/*
 * Copyright (c) 2009, Harald Kuhr
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name "TwelveMonkeys" nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.twelvemonkeys.imageio.metadata;

import com.twelvemonkeys.lang.Validate;
import com.twelvemonkeys.util.CollectionUtil;

import java.lang.reflect.Array;
import java.util.Arrays;

/**
 * AbstractEntry
 *
 * @author <a href="mailto:harald.kuhr@gmail.com">Harald Kuhr</a>
 * @author last modified by $Author: haraldk$
 * @version $Id: AbstractEntry.java,v 1.0 Nov 12, 2009 12:43:13 AM haraldk Exp$
 */
public abstract class AbstractEntry implements Entry {

    private final Object identifier;
    private final Object value; // Entries are immutable, directories can be mutated

    protected AbstractEntry(final Object identifier, final Object value) {
        Validate.notNull(identifier, "identifier");

        this.identifier = identifier;
        this.value = value;
    }

    public final Object getIdentifier() {
		return null;
	}

    /**
     * Returns a format-native identifier. 
     * For example {@code "2:00"} for IPTC "Record Version" field, or {@code "0x040c"} for PSD "Thumbnail" resource. 
     * This default implementation simply returns {@code String.valueOf(getIdentifier())}.
     * 
     * @return a format-native identifier.
     */
    protected String getNativeIdentifier() {
		return null;
	}

    /**
     * Returns {@code null}, meaning unknown or undefined.
     *
     * @return {@code null}.
     */
    public String getFieldName() {
		return null;
	}

    public Object getValue() {
		return null;
	}

    public String getValueAsString() {
		return null;
	}

    private static String arrayToString(final Object value) {
		return null;
	}

    public String getTypeName() {
		return null;
	}

    public int valueCount() {
		return 0;
	}

    /// Object

    @Override
    public int hashCode() {
		return 0;
	}

    @Override
    public boolean equals(final Object pOther) {
		return false;
	}

    private boolean valueEquals(final AbstractEntry other) {
		return false;
	}

    static boolean arrayEquals(final Object thisArray, final Object otherArray) {
		return false;
	}

    @Override
    public String toString() {
		return null;
	}
}
