/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.core.util.db;

import java.sql.Connection;
import java.sql.Driver;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.jcr.RepositoryException;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.commons.dbcp.BasicDataSource;
import org.apache.commons.dbcp.DelegatingConnection;
import org.apache.commons.pool.impl.GenericObjectPool;
import org.apache.jackrabbit.core.config.DataSourceConfig;
import org.apache.jackrabbit.core.config.DataSourceConfig.DataSourceDefinition;
import org.apache.jackrabbit.core.util.db.DataSourceWrapper;
import org.apache.jackrabbit.util.Base64;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * A factory for new database connections.
 * Supported are regular JDBC drivers, as well as
 * JNDI resources.
 *
 * FIXME: the registry currently is ClassLoader wide. I.e., if you start two repositories
 * then you share the registered datasources...
 */
public final class ConnectionFactory {

    private static final Logger log = LoggerFactory.getLogger(ConnectionFactory.class);

    /**
     * The lock to protect the fields of this class.
     */
    private final Object lock = new Object();

    /**
     * The data sources without logical name. The keys in the map are based on driver-url-user combination.
     */
    private final Map<String, DataSource> keyToDataSource = new HashMap<String, DataSource>();

    /**
     * The configured data sources with logical name. The keys in the map are the logical name.
     */
    private final Map<String, DataSource> nameToDataSource = new HashMap<String, DataSource>();

    /**
     * The configured data source defs. The keys in the map are the logical name.
     */
    private final Map<String, DataSourceDefinition> nameToDataSourceDef = new HashMap<String, DataSourceDefinition>();

    /**
     * The list of data sources created by this factory.
     */
    private final List<BasicDataSource> created = new ArrayList<BasicDataSource>();

    private boolean closed = false;

    /**
     * Registers a number of data sources.
     *
     * @param dsc the {@link DataSourceConfig} which contains the configuration
     */
    public void registerDataSources(DataSourceConfig dsc) throws RepositoryException {
	}

    /**
     * Retrieves a configured data source by logical name.
     *
     * @param logicalName the name of the {@code DataSource}
     * @return a {@code DataSource}
     * @throws RepositoryException if there is no {@code DataSource} with the given name
     */
    public DataSource getDataSource(String logicalName) throws RepositoryException {
		return null;
	}

    /**
     * @param logicalName the name of the {@code DataSource}
     * @return the configured database type
     * @throws RepositoryException if there is no {@code DataSource} with the given name
     */
    public String getDataBaseType(String logicalName) throws RepositoryException {
		return null;
	}

    /**
     * Retrieve a {@code DataSource} for the specified properties.
     * This can be a JNDI Data Source as well. To do that,
     * the driver class name must reference a {@code javax.naming.Context} class
     * (for example {@code javax.naming.InitialContext}), and the URL must be the JNDI URL
     * (for example {@code java:comp/env/jdbc/Test}).
     *
     * @param driver the JDBC driver or the Context class
     * @param url the database URL
     * @param user the user name
     * @param password the password
     * @return the {@code DataSource}
     * @throws RepositoryException if the driver could not be loaded
     * @throws SQLException if the connection could not be established
     */
    public DataSource getDataSource(String driver, String url, String user, String password)
            throws RepositoryException, SQLException    {
				return null;
			}

    /**
     *
     */
    public void close() {
	}

    /**
     * Needed for pre-10R2 Oracle blob support....:(
     *
     * This method actually assumes that we are using commons DBCP 1.2.2.
     *
     * @param con the commons-DBCP {@code DelegatingConnection} to unwrap
     * @return the unwrapped connection
     */
    public static Connection unwrap(Connection con) throws SQLException {
		return null;
	}

    private void sanityCheck() {
	}

    /**
     * Create a new pooling data source or finds an existing JNDI data source (depends on driver).
     *
     * @param driver
     * @param url
     * @param user
     * @param password
     * @return
     * @throws RepositoryException
     */
    private DataSource createDataSource(String driver, String url, String user, String password)
            throws RepositoryException {
				return null;
			}

    /**
     * Loads and returns the given JDBC driver (or JNDI context) class.
     * Returns <code>null</code> if a class name is not given.
     *
     * @param driver driver class name
     * @return driver class, or <code>null</code>
     * @throws RepositoryException if the class can not be loaded
     */
    private Class<?> getDriverClass(String driver)
            throws RepositoryException {
				return null;
			}

    /**
     * Returns the JDBC {@link DataSource} bound to the given name in
     * the JNDI {@link Context} identified by the given class.
     *
     * @param contextClass class that is instantiated to get the JNDI context
     * @param name name of the DataSource within the JNDI context
     * @return the DataSource bound in JNDI
     * @throws RepositoryException if the JNDI context can not be accessed,
     *                             or if the named DataSource is not found
     */
    private DataSource getJndiDataSource(
            Class<Context> contextClass, String name)
            throws RepositoryException {
				return null;
			}

    /**
     * Creates and returns a pooling JDBC {@link DataSource} for accessing
     * the database identified by the given driver class and JDBC
     * connection URL. The driver class can be <code>null</code> if
     * a specific driver has not been configured.
     *
     * @param driverClass the JDBC driver class, or <code>null</code>
     * @param url the JDBC connection URL
     * @return pooling DataSource for accessing the specified database
     */
    private BasicDataSource getDriverDataSource(
            Class<?> driverClass, String url, String user, String password) {
				return null;
			}

    private String guessValidationQuery(String url) {
		return null;
	}
}
