/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.jackrabbit.oak.plugins.index.lucene.util;

import java.util.Map;
import java.util.Set;

import javax.jcr.Node;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;

import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import org.apache.jackrabbit.oak.api.PropertyState;
import org.apache.jackrabbit.oak.api.Tree;
import org.apache.jackrabbit.oak.api.Type;
import org.apache.jackrabbit.oak.commons.PathUtils;
import org.apache.jackrabbit.oak.plugins.index.IndexConstants;
import org.apache.jackrabbit.oak.plugins.index.PathFilter;
import org.apache.jackrabbit.oak.plugins.index.lucene.LuceneIndexConstants;
import org.apache.jackrabbit.oak.plugins.tree.TreeFactory;
import org.apache.jackrabbit.oak.spi.state.EqualsDiff;
import org.apache.jackrabbit.oak.spi.state.NodeBuilder;
import org.apache.jackrabbit.oak.spi.state.NodeState;

import static com.google.common.base.Preconditions.checkNotNull;
import static java.util.Arrays.asList;
import static org.apache.jackrabbit.JcrConstants.JCR_PRIMARYTYPE;
import static org.apache.jackrabbit.JcrConstants.NT_UNSTRUCTURED;
import static org.apache.jackrabbit.oak.api.Type.NAME;
import static org.apache.jackrabbit.oak.api.Type.STRINGS;
import static org.apache.jackrabbit.oak.plugins.memory.EmptyNodeState.EMPTY_NODE;

public final class IndexDefinitionBuilder {
    private final NodeBuilder builder;
    private final Tree tree;
    private final Map<String, IndexRule> rules = Maps.newHashMap();
    private final Map<String, AggregateRule> aggRules = Maps.newHashMap();
    private final Tree indexRule;
    private final boolean autoManageReindexFlag;
    private Tree aggregatesTree;
    private final NodeState initial;
    private boolean reindexRequired;


    public IndexDefinitionBuilder(){
        this(EMPTY_NODE.builder());
    }

    public IndexDefinitionBuilder(NodeBuilder nodeBuilder){
        this(nodeBuilder, true);
    }

    public IndexDefinitionBuilder(NodeBuilder nodeBuilder, boolean autoManageReindexFlag){
        this.autoManageReindexFlag = autoManageReindexFlag;
        this.builder = nodeBuilder;
        this.initial = nodeBuilder.getNodeState();
        this.tree = TreeFactory.createTree(builder);
        tree.setProperty(LuceneIndexConstants.COMPAT_MODE, 2);
        tree.setProperty("async", "async");
        tree.setProperty(IndexConstants.TYPE_PROPERTY_NAME, "lucene");
        tree.setProperty(JCR_PRIMARYTYPE, "oak:QueryIndexDefinition", NAME);
        indexRule = getOrCreateChild(tree, LuceneIndexConstants.INDEX_RULES);
    }

    public IndexDefinitionBuilder evaluatePathRestrictions(){
		return null;
	}

    public IndexDefinitionBuilder includedPaths(String ... paths){
		return null;
	}

    public IndexDefinitionBuilder excludedPaths(String ... paths){
		return null;
	}

    public IndexDefinitionBuilder codec(String codecName){
		return null;
	}

    public IndexDefinitionBuilder noAsync(){
		return null;
	}

    public IndexDefinitionBuilder async(String ... asyncVals){
		return null;
	}

    public Tree getBuilderTree(){
		return null;
	}

    public NodeState build(){
		return null;
	}

    public Tree build(Tree tree){
		return null;
	}

    public Node build(Node node) throws RepositoryException {
		return null;
	}

    public boolean isReindexRequired() {
		return false;
	}

    private void setReindexFlagIfRequired(){
	}

    //~--------------------------------------< IndexRule >

    public IndexRule indexRule(String type){
		return null;
	}

    public boolean hasIndexRule(String type){
		return false;
	}

    public static class IndexRule {
        private final Tree indexRule;
        private final Tree propsTree;
        private final String ruleName;
        private final Map<String, PropertyRule> props = Maps.newHashMap();
        private final Set<String> propNodeNames = Sets.newHashSet();

        private IndexRule(Tree indexRule, String type) {
            this.indexRule = indexRule;
            this.propsTree = getOrCreateChild(indexRule, LuceneIndexConstants.PROP_NODE);
            this.ruleName = type;
            loadExisting();
        }

        public IndexRule indexNodeName(){
			return null;
		}

        public IndexRule includePropertyTypes(String ... types){
			return null;
		}

        public PropertyRule property(String name){
			return null;
		}

        public PropertyRule property(String name, boolean regex) {
			return null;
		}

        public PropertyRule property(String propDefnNodeName, String name) {
			return null;
		}

        public PropertyRule property(String propDefnNodeName, String name, boolean regex){
			return null;
		}

        private void loadExisting() {
		}

        private Tree findExisting(String name) {
			return null;
		}

        private String createPropNodeName(String name, boolean regex) {
			return null;
		}

        public String getRuleName() {
			return null;
		}

        public boolean hasPropertyRule(String propName){
			return false;
		}
    }

    public static class PropertyRule {
        private final IndexRule indexRule;
        private final Tree propTree;

        private PropertyRule(IndexRule indexRule, Tree propTree, String name, boolean regex) {
            this.indexRule = indexRule;
            this.propTree = propTree;
            propTree.setProperty(LuceneIndexConstants.PROP_NAME, name);
            if (regex) {
                propTree.setProperty(LuceneIndexConstants.PROP_IS_REGEX, true);
            }
        }

        public PropertyRule useInExcerpt(){
			return null;
		}

        public PropertyRule useInSpellcheck(){
			return null;
		}

        public PropertyRule type(String type){
			return null;
		}

        public PropertyRule useInSuggest(){
			return null;
		}

        public PropertyRule analyzed(){
			return null;
		}

        public PropertyRule nodeScopeIndex(){
			return null;
		}

        public PropertyRule ordered(){
			return null;
		}

        public PropertyRule ordered(String type){
			return null;
		}

        public PropertyRule disable() {
			return null;
		}

        public PropertyRule propertyIndex(){
			return null;
		}

        public PropertyRule nullCheckEnabled(){
			return null;
		}

        public PropertyRule notNullCheckEnabled(){
			return null;
		}

        public IndexRule enclosingRule(){
			return null;
		}

        public Tree getBuilderTree(){
			return null;
		}

        public PropertyRule property(String name){
			return null;
		}

        public PropertyRule property(String name, boolean regex) {
			return null;
		}

        public PropertyRule property(String propDefnNodeName, String name) {
			return null;
		}
    }

    //~--------------------------------------< Aggregates >

    public AggregateRule aggregateRule(String type){
		return null;
	}

    public AggregateRule aggregateRule(String primaryType, String ... includes){
		return null;
	}

    public static class AggregateRule {
        private final Tree aggregate;
        private final Map<String, Include> includes = Maps.newHashMap();

        private AggregateRule(Tree aggregate) {
            this.aggregate = aggregate;
            loadExisting(aggregate);
        }

        public Include include(String includePath) {
			return null;
		}

        private Tree findExisting(String includePath) {
			return null;
		}

        private void loadExisting(Tree aggregate) {
		}

        public static class Include {
            private final AggregateRule aggregateRule;
            private final Tree include;

            private Include(AggregateRule aggregateRule, Tree include) {
                this.aggregateRule = aggregateRule;
                this.include = include;
            }

            public Include path(String includePath) {
				return null;
			}

            public Include relativeNode(){
				return null;
			}

            public Include include(String path){
				return null;
			}

            public String getPath(){
				return null;
			}
        }
    }

    private static Tree getOrCreateChild(Tree tree, String name){
		return null;
	}

    static class SelectiveEqualsDiff extends EqualsDiff {
        public static boolean equals(NodeState before, NodeState after) {
			return false;
		}

        @Override
        public boolean propertyChanged(PropertyState before, PropertyState after) {
			return false;
		}

        private Set<String> getAsyncValuesWithoutNRT(PropertyState state){
			return null;
		}
    }

    static String getSafePropName(String relativePropName) {
		return null;
	}
}
