/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.oak.plugins.nodetype;

import java.util.List;
import java.util.Set;
import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;

import org.apache.jackrabbit.oak.api.PropertyState;
import org.apache.jackrabbit.oak.api.Type;
import org.apache.jackrabbit.oak.spi.state.ChildNodeEntry;
import org.apache.jackrabbit.oak.spi.state.NodeState;

import static com.google.common.base.Preconditions.checkNotNull;
import static com.google.common.collect.Iterables.addAll;
import static com.google.common.collect.Iterables.concat;
import static com.google.common.collect.Iterables.contains;
import static com.google.common.collect.Lists.newArrayListWithCapacity;
import static com.google.common.collect.Sets.newHashSet;
import static org.apache.jackrabbit.JcrConstants.JCR_DEFAULTPRIMARYTYPE;
import static org.apache.jackrabbit.JcrConstants.JCR_MANDATORY;
import static org.apache.jackrabbit.JcrConstants.JCR_MIXINTYPES;
import static org.apache.jackrabbit.JcrConstants.JCR_NODETYPENAME;
import static org.apache.jackrabbit.JcrConstants.JCR_PRIMARYTYPE;
import static org.apache.jackrabbit.JcrConstants.JCR_SAMENAMESIBLINGS;
import static org.apache.jackrabbit.JcrConstants.JCR_UUID;
import static org.apache.jackrabbit.oak.api.Type.UNDEFINED;
import static org.apache.jackrabbit.oak.api.Type.UNDEFINEDS;
import static org.apache.jackrabbit.oak.commons.PathUtils.dropIndexFromName;
import static org.apache.jackrabbit.oak.plugins.nodetype.NodeTypeConstants.REP_MANDATORY_CHILD_NODES;
import static org.apache.jackrabbit.oak.plugins.nodetype.NodeTypeConstants.REP_MANDATORY_PROPERTIES;
import static org.apache.jackrabbit.oak.plugins.nodetype.NodeTypeConstants.REP_NAMED_CHILD_NODE_DEFINITIONS;
import static org.apache.jackrabbit.oak.plugins.nodetype.NodeTypeConstants.REP_NAMED_PROPERTY_DEFINITIONS;
import static org.apache.jackrabbit.oak.plugins.nodetype.NodeTypeConstants.REP_RESIDUAL_CHILD_NODE_DEFINITIONS;
import static org.apache.jackrabbit.oak.plugins.nodetype.NodeTypeConstants.REP_RESIDUAL_PROPERTY_DEFINITIONS;
import static org.apache.jackrabbit.oak.plugins.nodetype.NodeTypeConstants.REP_SUPERTYPES;

class EffectiveType {

    private final List<NodeState> types;

    EffectiveType(@Nonnull List<NodeState> types) {
        this.types = checkNotNull(types);
    }

    /**
     * Checks whether this effective type contains the named type.
     *
     * @param name node type name
     * @return {@code true} if the named type is included,
     *         {@code false} otherwise
     */
    boolean isNodeType(@Nonnull String name) {
		return false;
	}

    boolean isMandatoryProperty(@Nonnull String name) {
		return false;
	}

    @Nonnull
    Set<String> getMandatoryProperties() {
		return null;
	}

    boolean isMandatoryChildNode(@Nonnull String name) {
		return false;
	}

    @Nonnull
    Set<String> getMandatoryChildNodes() {
		return null;
	}

    /**
     * Finds a matching definition for a property with the given name and type.
     *
     * @param property modified property
     * @return matching property definition, or {@code null}
     */
    @CheckForNull
    NodeState getDefinition(@Nonnull PropertyState property) {
		return null;
	}

    /**
     * Finds a matching definition for a child node with the given name and
     * types.
     *
     * @param nameWithIndex child node name, possibly with an SNS index
     * @param effective effective types of the child node
     * @return {@code true} if there's a matching child node definition,
     *         {@code false} otherwise
     */
    boolean isValidChildNode(@Nonnull String nameWithIndex, @Nonnull EffectiveType effective) {
		return false;
	}

    /**
     * Finds the default node type for a child node with the given name.
     *
     * @param nameWithIndex child node name, possibly with an SNS index
     * @return default type, or {@code null} if not found
     */
    @CheckForNull
    String getDefaultType(@Nonnull String nameWithIndex) {
		return null;
	}

    @Nonnull
    Set<String> getTypeNames() {
		return null;
	}

    //------------------------------------------------------------< Object >--

    @Override
    public String toString() {
		return null;
	}

    //-----------------------------------------------------------< private >--

    /**
     * Depending on the given SNS flag, checks whether the given child node
     * definition allows same-name-siblings.
     *
     * @param sns SNS flag, {@code true} if processing an SNS node
     * @param definition child node definition
     */
    private boolean snsMatch(boolean sns, @Nonnull NodeState definition) {
		return false;
	}

    private boolean nameSetContains(@Nonnull String set, @Nonnull String name) {
		return false;
	}

    @Nonnull
    private Set<String> getNameSet(@Nonnull String set) {
		return null;
	}

}
