/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.jackrabbit.oak.plugins.version;

import static com.google.common.base.Preconditions.checkNotNull;
import static org.apache.jackrabbit.JcrConstants.JCR_BASEVERSION;
import static org.apache.jackrabbit.JcrConstants.JCR_ISCHECKEDOUT;
import static org.apache.jackrabbit.JcrConstants.JCR_UUID;
import static org.apache.jackrabbit.oak.plugins.memory.EmptyNodeState.MISSING_NODE;
import static org.apache.jackrabbit.oak.plugins.version.VersionConstants.RESTORE_PREFIX;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.jcr.nodetype.NodeDefinition;
import javax.jcr.version.OnParentVersionAction;

import org.apache.jackrabbit.JcrConstants;
import org.apache.jackrabbit.oak.api.CommitFailedException;
import org.apache.jackrabbit.oak.api.PropertyState;
import org.apache.jackrabbit.oak.api.Type;
import org.apache.jackrabbit.oak.plugins.lock.LockConstants;
import org.apache.jackrabbit.oak.plugins.tree.TreeFactory;
import org.apache.jackrabbit.oak.spi.commit.CommitInfo;
import org.apache.jackrabbit.oak.spi.commit.Editor;
import org.apache.jackrabbit.oak.spi.state.NodeBuilder;
import org.apache.jackrabbit.oak.spi.state.NodeState;

/**
 * TODO document
 */
class VersionEditor implements Editor {

    private final VersionEditor parent;
    private final ReadWriteVersionManager vMgr;
    private final NodeBuilder node;
    private final String name;
    private Boolean isVersionable = null;
    private NodeState before;
    private NodeState after;
    private boolean isReadOnly;
    private CommitInfo commitInfo;

    public VersionEditor(@Nonnull NodeBuilder versionStore,
                         @Nonnull NodeBuilder workspaceRoot,
                         @Nonnull CommitInfo commitInfo) {
        this(null, new ReadWriteVersionManager(checkNotNull(versionStore),
                checkNotNull(workspaceRoot)), workspaceRoot, "", commitInfo);
    }

    VersionEditor(@Nullable VersionEditor parent,
                  @Nonnull ReadWriteVersionManager vMgr,
                  @Nonnull NodeBuilder node,
                  @Nonnull String name,
                  @Nonnull CommitInfo commitInfo) {
        this.parent = parent;
        this.vMgr = checkNotNull(vMgr);
        this.node = checkNotNull(node);
        this.name = checkNotNull(name);
        this.commitInfo = commitInfo;
    }

    @Override
    public void enter(NodeState before, NodeState after)
            throws CommitFailedException {
			}

    @Override
    public void leave(NodeState before, NodeState after)
            throws CommitFailedException {
			}

    @Override
    public void propertyAdded(PropertyState after)
            throws CommitFailedException {
			}

    @Override
    public void propertyChanged(PropertyState before, PropertyState after)
            throws CommitFailedException {
			}

    @Override
    public void propertyDeleted(PropertyState before)
            throws CommitFailedException {
			}

    @Override
    public Editor childNodeAdded(String name, NodeState after) {
		return null;
	}

    @Override
    public Editor childNodeChanged(String name, NodeState before,
            NodeState after) {
				return null;
			}

    @Override
    public Editor childNodeDeleted(String name, NodeState before) {
		return null;
	}

    /**
     * Returns {@code true} if the node of this VersionDiff is versionable;
     * {@code false} otherwise.
     *
     * @return whether the node is versionable.
     */
    private boolean isVersionable() {
		return false;
	}

    private boolean isVersionProperty(PropertyState state) {
		return false;
	}

    private boolean isLockProperty(PropertyState state) {
		return false;
	}

    /**
     * @return {@code true} if this node <b>was</b> checked in. That is,
     *         this method checks the before state for the jcr:isCheckedOut
     *         property.
     */
    private boolean wasCheckedIn() {
		return false;
	}

    private boolean hasNewIdentifier() {
		return false;
	}

    private StringBuilder buildBeforeIdentifier(StringBuilder identifier) {
		return null;
	}

    private StringBuilder buildAfterIdentifier(StringBuilder identifier) {
		return null;
	}

    private static void throwCheckedIn(String msg)
            throws CommitFailedException {
			}

    private static void throwProtected(String name)
            throws CommitFailedException {
			}

    private boolean isIgnoreOnOPV() throws CommitFailedException {
		return false;
	}

    private int getOPV(PropertyState property) throws CommitFailedException {
		return 0;
	}
}
