/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.rmi.server;

import java.rmi.RemoteException;

import javax.jcr.Value;
import javax.jcr.nodetype.NodeDefinition;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.PropertyDefinition;

import org.apache.jackrabbit.rmi.remote.RemoteIterator;
import org.apache.jackrabbit.rmi.remote.RemoteNodeDefinition;
import org.apache.jackrabbit.rmi.remote.RemoteNodeType;
import org.apache.jackrabbit.rmi.remote.RemotePropertyDefinition;

/**
 * Remote adapter for the JCR {@link javax.jcr.nodetype.NodeType NodeType}
 * interface. This class makes a local node type available as an RMI service
 * using the
 * {@link org.apache.jackrabbit.rmi.remote.RemoteNodeType RemoteNodeType}
 * interface.
 *
 * @see javax.jcr.nodetype.NodeType
 * @see org.apache.jackrabbit.rmi.remote.RemoteNodeType
 */
public class ServerNodeType extends ServerObject implements RemoteNodeType {

    /** The adapted local node type. */
    private NodeType type;

    /**
     * Creates a remote adapter for the given local node type.
     *
     * @param type local node type
     * @param factory remote adapter factory
     * @throws RemoteException on RMI errors
     */
    public ServerNodeType(NodeType type, RemoteAdapterFactory factory)
            throws RemoteException {
        super(factory);
        this.type = type;
    }

    /**
     * Utility method for creating an array of remote references for
     * local node definitions. The remote references are created using the
     * remote adapter factory.
     * <p>
     * A <code>null</code> input is treated as an empty array.
     *
     * @param defs local node definition array
     * @return remote node definition array
     * @throws RemoteException on RMI errors
     */
    private RemoteNodeDefinition[] getRemoteNodeDefArray(NodeDefinition[] defs)
            throws RemoteException {
				return null;
			}

    /**
     * Utility method for creating an array of remote references for
     * local property definitions. The remote references are created using the
     * remote adapter factory.
     * <p>
     * A <code>null</code> input is treated as an empty array.
     *
     * @param defs local property definition array
     * @return remote property definition array
     * @throws RemoteException on RMI errors
     */
    private RemotePropertyDefinition[] getRemotePropertyDefArray(
            PropertyDefinition[] defs) throws RemoteException {
				return null;
			}


    /** {@inheritDoc} */
    public String getName() throws RemoteException {
		return null;
	}

    /** {@inheritDoc} */
    public boolean isMixin() throws RemoteException {
		return false;
	}

    /** {@inheritDoc} */
    public boolean isAbstract() throws RemoteException {
		return false;
	}

    /** {@inheritDoc} */
    public boolean hasOrderableChildNodes() throws RemoteException {
		return false;
	}

    /** {@inheritDoc} */
    public RemoteNodeType[] getSupertypes() throws RemoteException {
		return null;
	}

    /** {@inheritDoc} */
    public RemoteNodeType[] getDeclaredSupertypes() throws RemoteException {
		return null;
	}

    /** {@inheritDoc} */
    public boolean isNodeType(String type) throws RemoteException {
		return false;
	}

    /** {@inheritDoc} */
    public RemotePropertyDefinition[] getPropertyDefs() throws RemoteException {
		return null;
	}

    /** {@inheritDoc} */
    public RemotePropertyDefinition[] getDeclaredPropertyDefs()
            throws RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteNodeDefinition[] getChildNodeDefs() throws RemoteException {
		return null;
	}

    /** {@inheritDoc} */
    public RemoteNodeDefinition[] getDeclaredChildNodeDefs() throws RemoteException {
		return null;
	}

    /** {@inheritDoc} */
    public boolean canSetProperty(String name, Value value)
            throws RemoteException {
				return false;
			}

    /** {@inheritDoc} */
    public boolean canSetProperty(String name, Value[] values)
            throws RemoteException {
				return false;
			}

    /** {@inheritDoc} */
    public boolean canAddChildNode(String name) throws RemoteException {
		return false;
	}

    /** {@inheritDoc} */
    public boolean canAddChildNode(String name, String type)
            throws RemoteException {
				return false;
			}

    /** {@inheritDoc} */
    public boolean canRemoveItem(String name) throws RemoteException {
		return false;
	}

    /** {@inheritDoc} */
    public String getPrimaryItemName() throws RemoteException {
		return null;
	}

    /** {@inheritDoc} */
	public boolean canRemoveNode(String nodeName) {
		return false;
	}

    /** {@inheritDoc} */
	public boolean canRemoveProperty(String propertyName) {
		return false;
	}

    /** {@inheritDoc} */
	public String[] getDeclaredSupertypeNames() {
		return null;
	}

    /** {@inheritDoc} */
	public boolean isQueryable() {
		return false;
	}

    /** {@inheritDoc} */
	public RemoteIterator getDeclaredSubtypes() throws RemoteException {
		return null;
	}

    /** {@inheritDoc} */
	public RemoteIterator getSubtypes() throws RemoteException {
		return null;
	}

}
