/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.spi.commons;

import org.apache.jackrabbit.spi.Event;
import org.apache.jackrabbit.spi.ItemId;
import org.apache.jackrabbit.spi.NodeId;
import org.apache.jackrabbit.spi.Name;
import org.apache.jackrabbit.spi.Path;
import org.apache.jackrabbit.spi.QValue;

import javax.jcr.RepositoryException;
import javax.jcr.UnsupportedRepositoryOperationException;
import java.io.Serializable;
import java.util.Arrays;
import java.util.Map;
import java.util.Collections;
import java.util.HashMap;

/**
 * <code>EventImpl</code> implements a serializable SPI
 * {@link org.apache.jackrabbit.spi.Event}.
 */
public class EventImpl implements Event, Serializable {

    /**
     * The SPI event type.
     * @see Event
     */
    private final int type;

    /**
     * The path of the affected item.
     */
    private final Path path;

    /**
     * The id of the affected item.
     */
    private final ItemId itemId;

    /**
     * The id of the affected item.
     */
    private final NodeId parentId;

    /**
     * The name of the primary node type of the 'associated' node of this event.
     */
    private final Name primaryNodeTypeName;

    /**
     * The names of the mixin types of the 'associated' node of this event.
     */
    private final Name[] mixinTypeNames;

    /**
     * The user ID connected with this event.
     */
    private final String userId;

    private final String userData;
    private final long timestamp;
    private final Map<Name, QValue> info;

    /**
     * Creates a new serializable event.
     * @deprecated
     */
    public EventImpl(int type, Path path, ItemId itemId, NodeId parentId,
                     Name primaryNodeTypeName, Name[] mixinTypeNames,
                     String userId) {
        this(type, path, itemId, parentId, primaryNodeTypeName, mixinTypeNames, userId, null, Long.MIN_VALUE, Collections.EMPTY_MAP);
    }

    /**
     * Creates a new serializable event.
     */
    public EventImpl(int type, Path path, ItemId itemId, NodeId parentId,
                     Name primaryNodeTypeName, Name[] mixinTypeNames,
                     String userId, String userData, long timestamp,
                     Map<Name, QValue> info) {
        this.type = type;
        this.path = path;
        this.itemId = itemId;
        this.parentId = parentId;
        this.primaryNodeTypeName = primaryNodeTypeName;
        this.mixinTypeNames = mixinTypeNames;
        this.userId = userId;

        this.userData = userData;
        this.info = new HashMap<Name, QValue>(info);
        this.timestamp = timestamp;
    }

    //--------------------------------------------------------------< Event >---
    /**
     * {@inheritDoc}
     */
    public int getType() {
		return 0;
	}

    /**
     * {@inheritDoc}
     */
    public Path getPath() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    public ItemId getItemId() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    public NodeId getParentId() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    public Name getPrimaryNodeTypeName() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    public Name[] getMixinTypeNames() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    public String getUserID() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    public Map<Name, QValue> getInfo() throws RepositoryException {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    public String getUserData() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    public long getDate() throws RepositoryException {
		return 0L;
	}

    //-------------------------------------------------------------< Object >---
    @Override
    public String toString() {
		return null;
	}
    
}
