/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.spi.commons.query.qom;

import org.apache.jackrabbit.spi.commons.conversion.NamePathResolver;
import org.apache.jackrabbit.spi.Name;

import javax.jcr.query.InvalidQueryException;

import java.util.Map;
import java.util.HashMap;
import java.util.Arrays;
import java.util.Iterator;

/**
 * <code>QueryObjectModelTree</code> implements the root node of an object
 * query model tree.
 */
public class QueryObjectModelTree extends AbstractQOMNode {

    /**
     * The node-tuple source for this query.
     */
    private final SourceImpl source;

    /**
     * The constraint for this query.
     */
    private final ConstraintImpl constraint;

    /**
     * The orderings for this query.
     */
    private final OrderingImpl[] orderings;

    /**
     * The columns for this query.
     */
    private final ColumnImpl[] columns;

    /**
     * All selectors available in this query object model. Key=Name
     */
    private final Map selectors = new HashMap();

    public QueryObjectModelTree(NamePathResolver resolver,
                                SourceImpl source,
                                ConstraintImpl constraint,
                                OrderingImpl[] orderings,
                                ColumnImpl[] columns)
            throws InvalidQueryException {
        super(resolver);
        this.source = source;
        this.constraint = constraint;
        this.orderings = orderings;
        this.columns = columns;
        for (Iterator it = Arrays.asList(source.getSelectors()).iterator(); it.hasNext(); ) {
            SelectorImpl selector = (SelectorImpl) it.next();
            if (selectors.put(selector.getSelectorQName(), selector) != null) {
                throw new InvalidQueryException("Duplicate selector name: " +
                        selector.getSelectorName());
            }
        }
        if (selectors.size() == 1) {
            // there is only one selector, which is also a default selector
            selectors.put(null, selectors.values().iterator().next());
        }
        checkQuery();
    }

    /**
     * Gets the node-tuple source for this query.
     *
     * @return the node-tuple source; non-null
     */
    public SourceImpl getSource() {
		return null;
	}

    /**
     * Gets the constraint for this query.
     *
     * @return the constraint, or null if none
     */
    public ConstraintImpl getConstraint() {
		return null;
	}

    /**
     * Gets the orderings for this query.
     *
     * @return an array of zero or more orderings; non-null
     */
    public OrderingImpl[] getOrderings() {
		return null;
	}

    /**
     * Gets the columns for this query.
     *
     * @return an array of zero or more columns; non-null
     */
    public ColumnImpl[] getColumns() {
		return null;
	}

    /**
     * Returns the selector with the given <code>name</code> or
     * <code>null</code> if there is no selector with this name.
     *
     * @param name the name of a selector.
     * @return the selector or <code>null</code> if there is no such selector.
     */
    public SelectorImpl getSelector(Name name) {
		return null;
	}

    //-----------------------< AbstractQOMNode >--------------------------------

    /**
     * Accepts a <code>visitor</code> and calls the appropriate visit method
     * depending on the type of this QOM node.
     *
     * @param visitor the visitor.
     */
    public Object accept(QOMTreeVisitor visitor, Object data) throws Exception {
		return null;
	}

    /**
     * Checks if this QOM is valid.
     *
     * @throws InvalidQueryException if the QOM is invalid.
     */
    private void checkQuery() throws InvalidQueryException {
	}

    //------------------------< Object >----------------------------------------

    public String toString() {
		return null;
	}


}
