/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.jackrabbit.vault.fs.io;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Nonnull;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.RepositoryException;

import org.apache.jackrabbit.vault.fs.api.PathFilterSet;
import org.apache.jackrabbit.vault.fs.api.VaultInputSource;
import org.apache.jackrabbit.vault.fs.config.ConfigurationException;
import org.apache.jackrabbit.vault.fs.config.DefaultMetaInf;
import org.apache.jackrabbit.vault.fs.config.DefaultWorkspaceFilter;
import org.apache.jackrabbit.vault.fs.config.MetaInf;
import org.apache.jackrabbit.vault.fs.config.VaultSettings;
import org.apache.jackrabbit.vault.util.Constants;
import org.apache.jackrabbit.vault.util.JcrConstants;
import org.apache.jackrabbit.vault.util.Text;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Implements an archive that is based on nt:file/nt:folder nodes in a JCR
 * repository.
 */
public class JcrArchive extends AbstractArchive {

    /**
     * default logger
     */
    private static final Logger log = LoggerFactory.getLogger(JcrArchive.class);

    private Node archiveRoot;

    private final String rootPath;

    private DefaultMetaInf inf;

    private Entry jcrRoot;

    // the 'chrooted' root. currently not used.
    private String chRoot;

    /**
     * Creates a new JCR archive rooted at the given node.
     * @param archiveRoot root node for this archive
     * @param rootPath root path
     */
    public JcrArchive(@Nonnull Node archiveRoot, @Nonnull String rootPath) {
        this.archiveRoot = archiveRoot;
        this.rootPath = rootPath;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void open(boolean strict) throws IOException {
	}

    private DefaultMetaInf loadMetaInf(Entry dir)
            throws IOException, ConfigurationException {
				return null;
			}

    /**
     * {@inheritDoc}
     */
    @Override
    public void close() {
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public Entry getJcrRoot() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public Entry getRoot() throws IOException {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public MetaInf getMetaInf() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public InputStream openInputStream(Entry entry) throws IOException {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public VaultInputSource getInputSource(final Entry entry) throws IOException {
		return null;
	}

    @Override
    public String toString() {
		return null;
	}

    /**
     * Implements a virtual entry for the intermediate directories
     */
    private static class VirtualEntry implements Entry {

        private final String name;

        private Map<String, Entry> children = new LinkedHashMap<String, Entry>();

        private VirtualEntry(String name) {
            this.name = name;
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public String getName() {
			return null;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        public boolean isDirectory() {
			return false;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        public Collection<? extends Entry> getChildren() {
			return null;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        public Entry getChild(String name) {
			return null;
		}
    }

    /**
     * Implements an archive entry that is based on a JCR node
     */
    private static class JcrEntry implements Entry {

        private final Node node;

        private final boolean isDir;

        private final String name;

        private JcrEntry(Node node, String name, boolean isDir) {
            this.node = node;
            this.isDir = isDir;
            this.name = name;
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public String getName() {
			return null;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        public boolean isDirectory() {
			return false;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        public Collection<Entry> getChildren() {
			return null;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        public Entry getChild(String name) {
			return null;
		}
    }

}