/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.vault.fs.io;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import org.apache.commons.io.IOUtils;
import org.apache.jackrabbit.vault.fs.api.VaultInputSource;
import org.apache.jackrabbit.vault.fs.config.DefaultMetaInf;
import org.apache.jackrabbit.vault.fs.config.MetaInf;
import org.apache.jackrabbit.vault.fs.config.VaultSettings;
import org.apache.jackrabbit.vault.util.Constants;
import org.apache.jackrabbit.vault.util.InputStreamPump;
import org.apache.jackrabbit.vault.util.Text;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Implements a input stream pump that analyzes the stream copies the stream content into memory.
 * The memory archive is initialized via the {@link #run(InputStream)}.
 */
public class MemoryArchive extends AbstractArchive implements InputStreamPump.Pump {

    /**
     * default logger
     */
    private static final Logger log = LoggerFactory.getLogger(MemoryArchive.class);

    private final VirtualEntry root;

    private final DefaultMetaInf inf;

    private boolean cacheMetaOnly = false;

    /**
     * Creates new memory archive.
     * @param metaOnly if {@code true} only the meta info content is cached.
     * @throws IOException if an I/O error occurrs
     */
    public MemoryArchive(boolean metaOnly) throws IOException {
        this.cacheMetaOnly = metaOnly;
        root = new VirtualEntry(null, "", 0, null);
        inf = new DefaultMetaInf();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void run(InputStream in) throws Exception {
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public void open(boolean strict) throws IOException {
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public InputStream openInputStream(Entry entry) throws IOException {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public VaultInputSource getInputSource(Entry entry) throws IOException {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public Entry getRoot() throws IOException {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public MetaInf getMetaInf() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public void close() {
	}

    /**
     * Implements a entry for this archive
     */
    private static class VirtualEntry implements Entry {

        private final VirtualEntry parent;

        private final String name;

        private final long time;

        private final byte[] data;

        private Map<String, VirtualEntry> children;

        private VirtualEntry(VirtualEntry parent, String name, long time, byte[] data) {
            this.parent = parent;
            this.name = name;
            this.time = time;
            this.data = data;
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public String getName() {
			return null;
		}

        @Nonnull
        public String getPath() {
			return null;
		}

        @Nonnull
        private StringBuilder getPath(@Nonnull StringBuilder sb) {
			return null;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        public boolean isDirectory() {
			return false;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        public Collection<? extends Entry> getChildren() {
			return null;
		}

        /**
         * {@inheritDoc}
         */
        @Override
        public Entry getChild(String name) {
			return null;
		}

        /**
         * Adds a new child entry.
         * @param name name
         * @param time timestamp
         * @param data data or {@code null}
         * @return the new entry
         */
        @Nonnull
        public VirtualEntry add(@Nonnull String name, long time, @Nullable byte[] data) {
			return null;
		}
    }

    /**
     * Safely returns the modification time of the zip entry or 0, if reading the time would
     * result in an error. for example due to http://bugs.java.com/view_bug.do?bug_id=JDK-8184940
     *
     * @param e the zip entry
     * @return the modification time
     */
    private static long safeGetTime(ZipEntry e) {
		return 0L;
	}
}