/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.jackrabbit.vault.fs.io;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import org.apache.commons.io.FileUtils;
import org.apache.jackrabbit.vault.fs.api.VaultInputSource;
import org.apache.jackrabbit.vault.fs.config.ConfigurationException;
import org.apache.jackrabbit.vault.fs.config.DefaultMetaInf;
import org.apache.jackrabbit.vault.fs.config.MetaInf;
import org.apache.jackrabbit.vault.fs.config.VaultSettings;
import org.apache.jackrabbit.vault.util.Constants;
import org.apache.jackrabbit.vault.util.Text;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Implements an archive that is based on a zip file.
 */
public class ZipArchive extends AbstractArchive {

    /**
     * default logger
     */
    private static final Logger log = LoggerFactory.getLogger(ZipArchive.class);

    /**
     * the zip file
     */
    private final File file;

    /**
     * {@code true} if file is temporary and can be deleted after this archive is closed.
     */
    private final boolean isTempFile;

    /**
     * The (loaded) meta info
     */
    private DefaultMetaInf inf;

    /**
     * the jar file that is created upon {@link #open(boolean)}
     */
    private JarFile jar;

    /**
     * the root entry of this archive
     */
    private EntryImpl root;

    /**
     * Creates a new archive that is based on the given zip file.
     * @param zipFile the zip file
     */
    public ZipArchive(@Nonnull File zipFile) {
        this(zipFile, false);
    }

    /**
     * Creates a new archive that is based on the given zip file.
     * @param zipFile the zip file
     * @param isTempFile if {@code true} if the file is considered temporary and can be deleted after this archive is closed.
     */
    public ZipArchive(@Nonnull File zipFile, boolean isTempFile) {
        this.file = zipFile;
        this.isTempFile = isTempFile;
    }

    @Override
    public void open(boolean strict) throws IOException {
	}

    @Override
    @Nullable
    public InputStream openInputStream(@Nullable Entry entry) throws IOException {
		return null;
	}

    @Override
    @Nullable
    public VaultInputSource getInputSource(@Nullable Entry entry) throws IOException {
		return null;
	}

    @Override
    public void close() {
	}

    @Override
    @Nonnull
    public Entry getRoot() throws IOException {
		return null;
	}

    @Override
    @Nonnull
    public MetaInf getMetaInf() {
		return null;
	}

    /**
     * Returns the underlying file or {@code null} if it does not exist.
     * @return the file or null.
     */
    @Nullable
    public File getFile() {
		return null;
	}

    /**
     * Returns the size of the underlying file or -1 if it does not exist.
     * @return the file size
     */
    public long getFileSize() {
		return 0L;
	}

    @Override
    public String toString() {
		return null;
	}

    /**
     * Implements the entry for this archive
     */
    private static class EntryImpl implements Entry {

        private final String name;

        private String zipEntryName;

        private final boolean isDirectory;

        private Map<String, EntryImpl> children;

        private EntryImpl(@Nonnull String name, boolean directory) {
            this.name = name;
            isDirectory = directory;
        }

        @Nonnull
        private EntryImpl add(@Nonnull EntryImpl e) {
			return null;
		}

        @Nonnull
        public EntryImpl add(@Nonnull String name, boolean isDirectory) {
			return null;
		}

        @Override
        @Nonnull
        public String getName() {
			return null;
		}

        @Override
        public boolean isDirectory() {
			return false;
		}

        @Override
        @Nonnull
        public Collection<? extends Entry> getChildren() {
			return null;
		}

        @Override
        @Nullable
        public Entry getChild(@Nonnull String name) {
			return null;
		}

    }
}