/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.sling.discovery.commons.providers;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.sling.discovery.ClusterView;
import org.apache.sling.discovery.InstanceDescription;
import org.apache.sling.discovery.commons.providers.util.PropertyNameHelper;

/**
 * Base implementation for the InstanceDescription interface.
 * <p>
 * Allows creation of the object with clusterview and/or properties null - to be
 * set later but before usage!
 * <p>
 */
public class DefaultInstanceDescription implements InstanceDescription {

    /** the cluster view of which this instance is part of **/
    private ClusterView clusterView;

    /** whether this instance is the leader in the cluster **/
    private boolean isLeader;

    /** whether this instance is the local/own one **/
    private boolean isLocal;

    /** the sling id of this instance **/
    private String slingId;

    /** the properties of this instance **/
    private Map<String, String> properties;

    public DefaultInstanceDescription(final DefaultClusterView clusterView,
            final boolean isLeader, final boolean isOwn, final String slingId,
            final Map<String, String> properties) {
        // slingId must not be null - clusterView and properties can be null though
        if (slingId == null || slingId.length() == 0) {
            throw new IllegalArgumentException("slingId must not be null");
        }
        this.isLeader = isLeader;
        this.isLocal = isOwn;
        this.slingId = slingId;
        this.properties = filterValidProperties(properties);
        if (clusterView != null) {
            clusterView.addInstanceDescription(this);
            if (this.clusterView == null) {
                throw new IllegalStateException(
                        "clusterView should have been set by now");
            }
        }
    }

    @Override
    public String toString() {
		return null;
	}

    @Override
    public int hashCode() {
		return 0;
	}

    @Override
    public boolean equals(final Object obj) {
		return false;
	}

    public ClusterView getClusterView() {
		return null;
	}

    /**
     * Sets the cluster on this instance
     * @param clusterView
     */
    void setClusterView(ClusterView clusterView) {
	}

    public boolean isLeader() {
		return false;
	}

    public boolean isLocal() {
		return false;
	}

    public String getSlingId() {
		return null;
	}

    public String getProperty(final String name) {
		return null;
	}

    public Map<String, String> getProperties() {
		return null;
	}
    
    /**
     * Sets the properties of this instance
     * @param properties
     */
    protected void setProperties(final Map<String, String> properties) {
	}

    /** SLING-2883 : filter (pass-through) valid properties only **/
	private Map<String, String> filterValidProperties(
			Map<String, String> rawProps) {
				return null;
			}

	/** for testing only! **/
    public void setProperty(String key, String value) {
	}
}
