/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.sling.provisioning.model;

import java.util.Enumeration;

/**
 * Allows to process a value. A new value is created and for each part in the model a process
 * method is called. Subclasses can overwrite those methods to inject specific behavior.
 * The processor itself does not change anything in the model.
 */
class ModelProcessor {

    /**
     * Creates a copy of the model and calls a process method for each part found in the model.
     * This allows to modify the parts content (e.g. replace variables), but not to add or remove parts.
     * @param model The base model.
     * @return The processed and copied model.
     */
    public final Model process(final Model model) {
		return null;
	}

    protected KeyValueMap<String> processVariables(KeyValueMap<String> variables, Feature newFeature) {
		return null;
	}

    protected Artifact processArtifact(Artifact artifact, Feature newFeature, RunMode newRunMode) {
		return null;
	}

    protected Configuration processConfiguration(Configuration config, Feature newFeature, RunMode newRunMode) {
		return null;
	}

    protected KeyValueMap<String> processSettings(KeyValueMap<String> settings, Feature newFeature, RunMode newRunMode) {
		return null;
	}

}
