/*******************************************************************************
 * Copyright 2016 Adobe Systems Incorporated
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package we.retail.core.productrelationships;


import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;

import com.adobe.cq.commerce.api.CommerceException;
import com.adobe.cq.commerce.api.CommerceSession;
import com.adobe.cq.commerce.api.Product;
import com.adobe.cq.commerce.api.ProductRelationship;
import com.adobe.cq.commerce.common.AbstractJcrProduct;
import com.adobe.cq.commerce.common.DefaultProductRelationship;
import com.day.cq.wcm.api.PageManager;

/**
 * A sample RelatedProductsCollector which matches based on tags.
 */
public class SimilarProductsCollector {
    /**
     * The type of relationships to generate.
     */
    protected String relationshipType;
    protected String relationshipTitle;

    /**
     * The source context of the relationships.
     */
    protected ResourceResolver resolver;
    protected PageManager pageManager;
    protected CommerceSession commerceSession;
    protected List<List<String>> matchTagSets;
    protected List<String> exclusionSKUs;

    /**
     * A map from SKU to relationship.  This allows us to recommend a particular product
     * only once even if multiple presentations are found for it.
     */
    protected Map<String, ProductRelationship> relationships;


    SimilarProductsCollector(ResourceResolver resolver, CommerceSession session, String relationshipType, String relationshipTitle,
                             List<Product> contextProducts) {
        this.resolver = resolver;
        this.pageManager = resolver.adaptTo(PageManager.class);
        this.commerceSession = session;

        this.relationshipType = relationshipType;
        this.relationshipTitle = relationshipTitle;

        matchTagSets = new ArrayList<List<String>>();
        exclusionSKUs = new ArrayList<String>();
        for (Product product : contextProducts) {
            List<String> matchTags = new ArrayList<String>();
            Collections.addAll(matchTags, product.getProperty("cq:tags", String[].class));
            matchTagSets.add(matchTags);

            try {
                exclusionSKUs.add(product.getBaseProduct().getSKU());
            } catch (CommerceException e) {
                exclusionSKUs.add(product.getSKU());    // not perfect, but better than nothing
            }
        }

        this.relationships = new HashMap<String, ProductRelationship>();
    }

    /**
     * Add a product relationship. Include 'price', 'matched-tags' and 'rank' as extra metadata.
     * 
     * @param product
     *            The product to add
     * @param matchedTags
     *            The list of matched tags
     * @throws CommerceException
     */
    protected void addProduct(Product product, List<String> matchedTags) throws CommerceException {
	}

    public void walk(Resource resource) throws CommerceException {
	}

    /**
     * Return the collected relationships, sorted by 'rank'.
     * 
     * @return the list of relationships, sorted by 'rank'
     */
    public List<ProductRelationship> getRelationships() {
		return null;
	}
}


